@@ -1,8 +1,8 @@
 from __future__ import division
-import sys, re
-from cbook import iterable, flatten
-from transforms import identity_transform
-import matplotlib.units as units
+import re, warnings
+import matplotlib.cbook as cbook
+from transforms import Bbox, IdentityTransform, TransformedBbox, TransformedPath
+from path import Path
 
 ## Note, matplotlib artists use the doc strings for set and get
 # methods to enable the introspection methods of setp and getp.  Every
@@ -21,9 +21,10 @@ import matplotlib.units as units
 # http://groups.google.com/groups?hl=en&lr=&threadm=mailman.5090.1098044946.5135.python-list%40python.org&rnum=1&prev=/groups%3Fq%3D__doc__%2Bauthor%253Ajdhunter%2540ace.bsd.uchicago.edu%26hl%3Den%26btnG%3DGoogle%2BSearch
 
 
-class Artist:
+class Artist(object):
     """
-    Abstract base class for someone who renders into a FigureCanvas
+    Abstract base class for someone who renders into a
+    :class:`FigureCanvas`.
     """
 
     aname = 'Artist'
@@ -52,12 +53,15 @@ class Artist:
 
     def remove(self):
         """
-        Remove the artist from the figure if possible.  The effect will not
-        be visible until the figure is redrawn, e.g., with ax.draw_idle().
-        Call ax.relim() to update the axes limits if desired.
+        Remove the artist from the figure if possible.  The effect
+        will not be visible until the figure is redrawn, e.g., with
+        :meth:`matplotlib.axes.Axes.draw_idle`.  Call
+        :meth:`matplotlib.axes.Axes.relim` to update the axes limits
+        if desired.
 
-        Note: relim() will not see collections even if the collection
-        was added to axes with autolim=True.
+        Note: :meth:`~matplotlib.axes.Axes.relim` will not see
+        collections even if the collection was added to axes with
+        *autolim* = True.
 
         Note: there is no support for removing the artist's legend entry.
         """
@@ -134,18 +138,20 @@ class Artist:
 
     def set_transform(self, t):
         """
-        set the Transformation instance used by this artist
-
-        ACCEPTS: a matplotlib.transform transformation instance
+        Set the :class:`~matplotlib.transforms.Transform` instance
+        used by this artist.
         """
         self._transform = t
         self._transformSet = True
         self.pchanged()
 
     def get_transform(self):
-        'return the Transformation instance used by this artist'
+        """
+        Return the :class:`~matplotlib.transforms.Transform`
+        instance used by this artist.
+        """
         if self._transform is None:
-            self._transform = identity_transform()
+            self._transform = IdentityTransform()
         return self._transform
 
     def hitlist(self,event):
@@ -173,13 +179,13 @@ class Artist:
         """
         if callable(self._contains): return self._contains(self,mouseevent)
         #raise NotImplementedError,str(self.__class__)+" needs 'contains' method"
-        print str(self.__class__)+" needs 'contains' method"
+        warnings.warn("'%s' needs 'contains' method" % self.__class__.__name__)
         return False,{}
 
     def set_contains(self,picker):
         """Replace the contains test used by this artist. The new picker should
         be a callable function which determines whether the artist is hit by the
-        mouse event:
+        mouse event::
 
             hit, props = picker(artist, mouseevent)
 
@@ -228,26 +234,26 @@ class Artist:
           None -  picking is disabled for this artist (default)
 
           boolean - if True then picking will be enabled and the
-            artist will fire a pick event if the mouse event is over
-            the artist
+          artist will fire a pick event if the mouse event is over
+          the artist
 
           float - if picker is a number it is interpreted as an
-            epsilon tolerance in points and the the artist will fire
-            off an event if it's data is within epsilon of the mouse
-            event.  For some artists like lines and patch collections,
-            the artist may provide additional data to the pick event
-            that is generated, eg the indices of the data within
-            epsilon of the pick event
+          epsilon tolerance in points and the the artist will fire
+          off an event if it's data is within epsilon of the mouse
+          event.  For some artists like lines and patch collections,
+          the artist may provide additional data to the pick event
+          that is generated, eg the indices of the data within
+          epsilon of the pick event
 
           function - if picker is callable, it is a user supplied
-            function which determines whether the artist is hit by the
-            mouse event.
+          function which determines whether the artist is hit by the
+          mouse event::
 
               hit, props = picker(artist, mouseevent)
 
-            to determine the hit test.  if the mouse event is over the
-            artist, return hit=True and props is a dictionary of
-            properties you want added to the PickEvent attributes
+          to determine the hit test.  if the mouse event is over the
+          artist, return hit=True and props is a dictionary of
+          properties you want added to the PickEvent attributes.
 
         ACCEPTS: [None|float|boolean|callable]
         """
@@ -262,12 +268,16 @@ class Artist:
         return self.figure is not None
 
     def get_figure(self):
-        'return the figure instance'
+        """
+        Return the :class:`~matplotlib.figure.Figure` instance the
+        artist belongs to.
+        """
         return self.figure
 
     def set_figure(self, fig):
         """
-        Set the figure instance the artist belong to
+        Set the :class:`~matplotlib.figure.Figure` instance the artist
+        belongs to.
 
         ACCEPTS: a matplotlib.figure.Figure instance
         """
@@ -284,17 +294,54 @@ class Artist:
         self._clipon = clipbox is not None or self._clippath is not None
         self.pchanged()
 
-    def set_clip_path(self, path):
+    def set_clip_path(self, path, transform=None):
         """
-        Set the artist's clip path
+        Set the artist's clip path, which may be:
 
-        ACCEPTS: an agg.path_storage instance
+          a) a :class:`~matplotlib.patches.Patch` (or subclass) instance
+
+          b) a :class:`~matplotlib.path.Path` instance, in which case
+             an optional :class:`~matplotlib.transforms.Transform`
+             instance may be provided, which will be applied to the
+             path before using it for clipping.
+
+          c) *None*, to remove the clipping path
+
+        For efficiency, if the path happens to be an axis-aligned
+        rectangle, this method will set the clipping box to the
+        corresponding rectangle and set the clipping path to *None*.
+
+        ACCEPTS: a Path instance and a Transform instance, a Patch
+        instance, or None
         """
-        self._clippath = path
-        self._clipon = self.clipbox is not None or path is not None
+        from patches import Patch, Rectangle
+
+        success = False
+        if transform is None:
+            if isinstance(path, Rectangle):
+                self.clipbox = TransformedBbox(Bbox.unit(), path.get_transform())
+                self._clippath = None
+                success = True
+            elif isinstance(path, Patch):
+                self._clippath = TransformedPath(
+                    path.get_path(),
+                    path.get_transform())
+                success = True
+
+        if path is None:
+            self._clippath = None
+            success = True
+        elif isinstance(path, Path):
+            self._clippath = TransformedPath(path, transform)
+            success = True
+
+        if not success:
+            print type(path), type(transform)
+            raise TypeError("Invalid arguments to set_clip_path")
+
+        self._clipon = self.clipbox is not None or self._clippath is not None
         self.pchanged()
 
-
     def get_alpha(self):
         """
         Return the alpha value used for blending - not supported on all
@@ -322,6 +369,15 @@ class Artist:
         'Return artist clip path'
         return self._clippath
 
+    def get_transformed_clip_path_and_affine(self):
+        '''
+        Return the clip path with the non-affine part of its transformation applied,
+        and the remaining affine part of its transformation.
+        '''
+        if self._clippath is not None:
+            return self._clippath.get_transformed_path_and_affine()
+        return None, None
+
     def set_clip_on(self, b):
         """
         Set  whether artist uses clipping
@@ -337,7 +393,7 @@ class Artist:
     def _set_gc_clip(self, gc):
         'set the clip properly for the gc'
         if self.clipbox is not None:
-            gc.set_clip_rectangle(self.clipbox.get_bounds())
+            gc.set_clip_rectangle(self.clipbox)
         gc.set_clip_path(self._clippath)
 
     def draw(self, renderer, *args, **kwargs):
@@ -424,13 +480,14 @@ class Artist:
         self.pchanged()
 
     def update_from(self, other):
-        'copy properties from other to self'
+        'Copy properties from *other* to *self*.'
         self._transform = other._transform
         self._transformSet = other._transformSet
         self._visible = other._visible
         self._alpha = other._alpha
         self.clipbox = other.clipbox
         self._clipon = other._clipon
+        self._clippath = other._clippath
         self._lod = other._lod
         self._label = other._label
         self.pchanged()
@@ -451,26 +508,33 @@ class Artist:
 
 class ArtistInspector:
     """
-    A helper class to inspect an Artist and return information about
-    it's settable properties and their current values
+    A helper class to inspect an :class:`~matplotlib.artist.Artist`
+    and return information about it's settable properties and their
+    current values.
     """
     def __init__(self, o):
         """
-        Initialize the artist inspector with an artist or sequence of
-        artists.  Id a sequence is used, we assume it is a homogeneous
-        sequence (all Artists are of the same type) and it is your
-        responsibility to make sure this is so.
+        Initialize the artist inspector with an
+        :class:`~matplotlib.artist.Artist` or sequence of
+        :class:`Artists`.  If a sequence is used, we assume it is a
+        homogeneous sequence (all :class:`Artists` are of the same
+        type) and it is your responsibility to make sure this is so.
         """
-        if iterable(o) and len(o): o = o[0]
+        if cbook.iterable(o) and len(o): o = o[0]
         self.o = o
         self.aliasd = self.get_aliases()
 
     def get_aliases(self):
         """
-        get a dict mapping fullname -> alias for each alias in o.
-        Eg for lines: {'markerfacecolor': 'mfc',
-                       'linewidth'      : 'lw',
-                       }
+        Get a dict mapping *fullname* -> *alias* for each alias in the
+        :class:`~matplotlib.artist.ArtistInspector`.
+
+        Eg., for lines::
+
+          {'markerfacecolor': 'mfc',
+           'linewidth'      : 'lw',
+          }
+
         """
         names = [name for name in dir(self.o) if
                  (name.startswith('set_') or name.startswith('get_'))
@@ -487,12 +551,12 @@ class ArtistInspector:
     _get_valid_values_regex = re.compile(r"\n\s*ACCEPTS:\s*(.*)\n")
     def get_valid_values(self, attr):
         """
-        get the legal arguments for the setter associated with attr
+        Get the legal arguments for the setter associated with *attr*.
 
-        This is done by querying the doc string of the function set_attr
+        This is done by querying the docstring of the function set_ *attr*
         for a line that begins with ACCEPTS:
 
-        Eg, for a line linestyle, return
+        Eg., for a line linestyle, return
         [ '-' | '--' | '-.' | ':' | 'steps' | 'None' ]
         """
 
@@ -514,8 +578,8 @@ class ArtistInspector:
 
     def get_setters(self):
         """
-        Get the attribute strings with setters for object h.  Eg, for a line,
-        return ['markerfacecolor', 'linewidth', ....]
+        Get the attribute strings with setters for object.  Eg., for a line,
+        return ``['markerfacecolor', 'linewidth', ....]``.
         """
 
         setters = []
@@ -529,7 +593,10 @@ class ArtistInspector:
         return setters
 
     def is_alias(self, o):
-        'return true if method object o is an alias for another function'
+        """
+        Return *True* if method object *o* is an alias for another
+        function.
+        """
         ds = o.__doc__
         if ds is None: return False
         return ds.startswith('alias for ')
@@ -549,12 +616,12 @@ class ArtistInspector:
 
     def pprint_setters(self, prop=None, leadingspace=2):
         """
-        if prop is None, return a list of strings of all settable properies
-        and their valid values
+        If *prop* is *None*, return a list of strings of all settable properies
+        and their valid values.
 
-        if prop is not None, it is a valid property name and that
+        If *prop* is not *None*, it is a valid property name and that
         property will be returned as a string of property : valid
-        values
+        values.
         """
         if leadingspace:
             pad = ' '*leadingspace
@@ -568,6 +635,28 @@ class ArtistInspector:
         attrs.sort()
         lines = []
 
+        ########
+        names = [self.aliased_name(prop) for prop in attrs]
+        accepts = [self.get_valid_values(prop) for prop in attrs]
+
+        col0_len = max([len(n) for n in names])
+        col1_len = max([len(a) for a in accepts])
+        table_formatstr = pad + '='*col0_len + '   ' + '='*col1_len
+
+        lines.append('')
+        lines.append(table_formatstr)
+        lines.append(pad + 'Property'.ljust(col0_len+3) + \
+                     'Description'.ljust(col1_len))
+        lines.append(table_formatstr)
+
+        lines.extend([pad + n.ljust(col0_len+3) + a.ljust(col1_len)
+                      for n, a in zip(names, accepts)])
+
+        lines.append(table_formatstr)
+        lines.append('')
+        return lines
+        ########
+
         for prop in attrs:
             accepts = self.get_valid_values(prop)
             name = self.aliased_name(prop)
@@ -577,7 +666,7 @@ class ArtistInspector:
 
     def pprint_getters(self):
         """
-        return the getters and actual values as list of strings'
+        Return the getters and actual values as list of strings.
         """
         getters = [name for name in dir(self.o)
                    if name.startswith('get_')
@@ -600,6 +689,8 @@ class ArtistInspector:
 
 def getp(o, *args):
     """
+    .. TODO: What are 's' and 'h' arguments described below?
+
     Return the value of handle property s
 
     h is an instance of a class, eg a Line2D or an Axes or Text.
@@ -636,44 +727,45 @@ get.__doc__ = getp.__doc__
 
 def setp(h, *args, **kwargs):
     """
-    matplotlib supports the use of setp ("set property") and getp to set
-    and get object properties, as well as to do introspection on the
-    object For example, to set the linestyle of a line to be dashed, you
-    can do
+    matplotlib supports the use of :func:`setp` ("set property") and
+    :func:`getp` to set and get object properties, as well as to do
+    introspection on the object.  For example, to set the linestyle of a
+    line to be dashed, you can do::
 
       >>> line, = plot([1,2,3])
       >>> setp(line, linestyle='--')
 
     If you want to know the valid types of arguments, you can provide the
-    name of the property you want to set without a value
+    name of the property you want to set without a value::
 
       >>> setp(line, 'linestyle')
           linestyle: [ '-' | '--' | '-.' | ':' | 'steps' | 'None' ]
 
     If you want to see all the properties that can be set, and their
-    possible values, you can do
-
+    possible values, you can do::
 
       >>> setp(line)
-          ... long output listing omitted'
+          ... long output listing omitted
 
-    setp operates on a single instance or a list of instances.  If you
-    are in query mode introspecting the possible values, only the first
-    instance in the sequence is used.  When actually setting values,
-    all the instances will be set.  Eg, suppose you have a list of two
-    lines, the following will make both lines thicker and red
+    :func:`setp` operates on a single instance or a list of instances.
+    If you are in query mode introspecting the possible values, only
+    the first instance in the sequence is used.  When actually setting
+    values, all the instances will be set.  Eg., suppose you have a
+    list of two lines, the following will make both lines thicker and
+    red::
 
-        >>> x = arange(0,1.0,0.01)
-        >>> y1 = sin(2*pi*x)
-        >>> y2 = sin(4*pi*x)
-        >>> lines = plot(x, y1, x, y2)
-        >>> setp(lines, linewidth=2, color='r')
+      >>> x = arange(0,1.0,0.01)
+      >>> y1 = sin(2*pi*x)
+      >>> y2 = sin(4*pi*x)
+      >>> lines = plot(x, y1, x, y2)
+      >>> setp(lines, linewidth=2, color='r')
 
-    setp works with the matlab(TM) style string/value pairs or with
-    python kwargs.  For example, the following are equivalent
+    :func:`setp` works with the matlab(TM) style string/value pairs or
+    with python kwargs.  For example, the following are equivalent
 
-        >>> setp(lines, 'linewidth', 2, 'color', r')  # matlab style
-        >>> setp(lines, linewidth=2, color='r')       # python style
+      >>> setp(lines, 'linewidth', 2, 'color', r')  # matlab style
+
+      >>> setp(lines, linewidth=2, color='r')       # python style
     """
 
     insp = ArtistInspector(h)
@@ -686,8 +778,8 @@ def setp(h, *args, **kwargs):
         print insp.pprint_setters(prop=args[0])
         return
 
-    if not iterable(h): h = [h]
-    else: h = flatten(h)
+    if not cbook.iterable(h): h = [h]
+    else: h = cbook.flatten(h)
 
 
     if len(args)%2:
@@ -705,10 +797,10 @@ def setp(h, *args, **kwargs):
             funcName = "set_%s"%s
             func = getattr(o,funcName)
             ret.extend( [func(val)] )
-    return [x for x in flatten(ret)]
+    return [x for x in cbook.flatten(ret)]
 
 def kwdoc(a):
-    return '\n'.join(ArtistInspector(a).pprint_setters(leadingspace=4))
+    return '\n'.join(ArtistInspector(a).pprint_setters(leadingspace=2))
 
 kwdocd = dict()
 kwdocd['Artist'] = kwdoc(Artist)
