@@ -1,25 +1,21 @@
-import sys
+
 import matplotlib
-import time
+from matplotlib.rcsetup import interactive_bk
+from matplotlib.rcsetup import non_interactive_bk
+from matplotlib.rcsetup import all_backends
+from matplotlib.rcsetup import validate_backend
 
 __all__ = ['backend','show','draw_if_interactive',
            'new_figure_manager', 'backend_version']
 
-interactive_bk = ['GTK', 'GTKAgg', 'GTKCairo', 'FltkAgg', 'QtAgg', 'Qt4Agg', 
-                  'TkAgg', 'WX', 'WXAgg', 'CocoaAgg', 'Aqt']
-non_interactive_bk = ['Agg2', 'Agg', 'Cairo', 'EMF', 'GD', 'GDK', 'Paint', 
-                      'Pdf', 'PS', 'SVG', 'Template']
-all_backends = interactive_bk + non_interactive_bk
-
-backend = matplotlib.get_backend()
-if backend not in all_backends:
-    raise ValueError, 'Unrecognized backend %s' % backend
+backend = matplotlib.get_backend() # validates, to match all_backends
 
 def pylab_setup():
     'return new_figure_manager, draw_if_interactive and show for pylab'
     # Import the requested backend into a generic module object
 
-    backend_name = 'backend_'+backend.lower()
+    backend_name = 'backend_'+backend
+    backend_name = backend_name.lower() # until we banish mixed case
     backend_mod = __import__('matplotlib.backends.'+backend_name,
                              globals(),locals(),[backend_name])
 
@@ -42,7 +38,7 @@ def pylab_setup():
 
     # Additional imports which only happen for certain backends.  This section
     # should probably disappear once all backends are uniform.
-    if backend in ['WX','WXAgg']:
+    if backend.lower() in ['wx','wxagg']:
         Toolbar = backend_mod.Toolbar
         __all__.append('Toolbar')
 
@@ -50,7 +46,4 @@ def pylab_setup():
 
     return new_figure_manager, draw_if_interactive, show
 
-# a hack to keep old versions of ipython working with mpl
-if 'IPython.Shell' in  sys.modules:
-    new_figure_manager, draw_if_interactive, show = pylab_setup()
 
