@@ -47,7 +47,7 @@ def new_figure_manager(num, *args, **kwargs):
 class FigureCanvasGTKAgg(FigureCanvasGTK, FigureCanvasAgg):
     filetypes = FigureCanvasGTK.filetypes.copy()
     filetypes.update(FigureCanvasAgg.filetypes)
-    
+
     def configure_event(self, widget, event=None):
 
         if DEBUG: print 'FigureCanvasGTKAgg.configure_event'
@@ -61,7 +61,7 @@ class FigureCanvasGTKAgg(FigureCanvasGTK, FigureCanvasAgg):
         if w==1 or h==1: return # empty fig
 
         # compute desired figure size in inches
-        dpival = self.figure.dpi.get()
+        dpival = self.figure.dpi
         winch = w/dpival
         hinch = h/dpival
         self.figure.set_size_inches(winch, hinch)
@@ -80,6 +80,14 @@ class FigureCanvasGTKAgg(FigureCanvasGTK, FigureCanvasAgg):
         ren = self.get_renderer()
         w = int(ren.width)
         h = int(ren.height)
+
+        # There apparently is a bug here on some versions of pygtk
+        # (2.6) that crops up in corner cases.  Eg, in
+        # figimage_demo.py, there is background pixel noise because
+        # the figure frame is off and the blended image background has
+        # alpha 0. and you see the uninitialzed data ("screen noise").
+        # If in _image.cpp Image:from_image you fill the background
+        # with a non transparent value, it goes away.
         pixbuf = gtk.gdk.pixbuf_new_from_data(
             buf, gtk.gdk.COLORSPACE_RGB,  True, 8, w, h, w*4)
         pixmap.draw_pixbuf(pixmap.new_gc(), pixbuf, 0, 0, 0, 0, w, h,
