@@ -32,7 +32,7 @@ The files that are most relevant to backend_writers are
   matplotlib/backends/__init__.py
   matplotlib/__init__.py
   matplotlib/_pylab_helpers.py
-  
+
 Naming Conventions
 
   * classes Upper or MixedUpperCase
@@ -49,7 +49,6 @@ import matplotlib
 from matplotlib._pylab_helpers import Gcf
 from matplotlib.backend_bases import RendererBase, GraphicsContextBase,\
      FigureManagerBase, FigureCanvasBase
-from matplotlib.cbook import enumerate
 from matplotlib.figure import Figure
 from matplotlib.transforms import Bbox
 
@@ -62,39 +61,51 @@ class RendererTemplate(RendererBase):
     writing a new backend. Refer to backend_bases.RendererBase for
     documentation of the classes methods.
     """
-    def draw_arc(self, gc, rgbFace, x, y, width, height, angle1, angle2, rotation):
-        pass
-    
-    def draw_image(self, x, y, im, bbox):
-        pass
-    
-    def draw_line(self, gc, x1, y1, x2, y2):
-        pass
-    
-    def draw_lines(self, gc, x, y):
+    def __init__(self, dpi):
+        self.dpi = dpi
+
+    def draw_path(self, gc, path, transform, rgbFace=None):
         pass
 
-    def draw_point(self, gc, x, y):
+    # draw_markers is optional, and we get more correct relative
+    # timings by leaving it out.  backend implementers concerned with
+    # performance will probably want to implement it
+#     def draw_markers(self, gc, marker_path, marker_trans, path, trans, rgbFace=None):
+#         pass
+
+    # draw_path_collection is optional, and we get more correct
+    # relative timings by leaving it out. backend implementers concerned with
+    # performance will probably want to implement it
+#     def draw_path_collection(self, master_transform, cliprect, clippath,
+#                              clippath_trans, paths, all_transforms, offsets,
+#                              offsetTrans, facecolors, edgecolors, linewidths,
+#                              linestyles, antialiaseds):
+#         pass
+
+    # draw_quad_mesh is optional, and we get more correct
+    # relative timings by leaving it out.  backend implementers concerned with
+    # performance will probably want to implement it
+#     def draw_quad_mesh(self, master_transform, cliprect, clippath,
+#                        clippath_trans, meshWidth, meshHeight, coordinates,
+#                        offsets, offsetTrans, facecolors, antialiased,
+#                        showedges):
+#         pass
+
+    def draw_image(self, x, y, im, bbox, clippath=None, clippath_trans=None):
         pass
 
-    def draw_polygon(self, gcEdge, rgbFace, points):
+    def draw_text(self, gc, x, y, s, prop, angle, ismath=False):
         pass
 
-    def draw_rectangle(self, gcEdge, rgbFace, x, y, width, height):
-        pass
-
-    def draw_text(self, gc, x, y, s, prop, angle, ismath=False):    
-        pass
-         
     def flipy(self):
         return True
-    
+
     def get_canvas_width_height(self):
         return 100, 100
 
-    def get_text_width_height(self, s, prop, ismath):
-        return 1, 1
-                              
+    def get_text_width_height_descent(self, s, prop, ismath):
+        return 1, 1, 1
+
     def new_gc(self):
         return GraphicsContextTemplate()
 
@@ -129,10 +140,10 @@ class GraphicsContextTemplate(GraphicsContextBase):
     """
     pass
 
-        
-        
+
+
 ########################################################################
-#    
+#
 # The following functions and classes are for pylab and implement
 # window/figure managers, etc...
 #
@@ -194,47 +205,28 @@ class FigureCanvasTemplate(FigureCanvasBase):
         """
         Draw the figure using the renderer
         """
-        renderer = RendererTemplate()
+        renderer = RendererTemplate(self.figure.dpi)
         self.figure.draw(renderer)
-        
-    def print_figure(self, filename, dpi=None, facecolor='w', edgecolor='w',
-                     orientation='portrait', **kwargs):
+
+    # You should provide a print_xxx function for every file format
+    # you can write.
+
+    # If the file type is not in the base set of filetypes,
+    # you should add it to the class-scope filetypes dictionary as follows:
+    filetypes = FigureCanvasBase.filetypes.copy()
+    filetypes['foo'] = 'My magic Foo format'
+
+    def print_foo(self, filename, *args, **kwargs):
         """
-        Render the figure to hardcopy. Set the figure patch face and edge
-        colors.  This is useful because some of the GUIs have a gray figure
-        face color background and you'll probably want to override this on
-        hardcopy.
-
-        orientation - only currently applies to PostScript printing.
-
-        A GUI backend should save and restore the original figure settings.
-        An image backend does not need to do this since after the print the
-        figure is done
+        Write out format foo.  The dpi, facecolor and edgecolor are restored
+        to their original values after this call, so you don't need to
+        save and restore them.
         """
+        pass
 
-        if dpi is None: dpi = rcParams['savefig.dpi']
-        # save the figure settings, GUI backends only
-        #origDPI = self.figure.dpi.get()
-        #origfacecolor = self.figure.get_facecolor()
-        #origedgecolor = self.figure.get_edgecolor()
+    def get_default_filetype(self):
+        return 'foo'
 
-        # set the new parameters
-        self.figure.dpi.set(dpi)
-        self.figure.set_facecolor(facecolor)
-        self.figure.set_edgecolor(edgecolor)        
-
-        renderer = RendererTemplate()
-        self.figure.draw(renderer)
-        # do something to save to hardcopy
-
-        # restore original figure settings, GUI backends only
-        #self.figure.dpi.set(origDPI)
-        #self.figure.set_facecolor(origfacecolor)
-        #self.figure.set_edgecolor(origedgecolor)
-        # redraw the screen if necessary
-        #self.draw()
-        
-    
 class FigureManagerTemplate(FigureManagerBase):
     """
     Wrap everything up into a window for the pylab interface
@@ -244,9 +236,9 @@ class FigureManagerTemplate(FigureManagerBase):
     pass
 
 ########################################################################
-#    
+#
 # Now just provide the standard names that backend.__init__ is expecting
-# 
+#
 ########################################################################
 
 
