@@ -92,9 +92,10 @@ Examples which work on this release:
  (3) - Clipping seems to be broken.
 """
 
-cvs_id = '$Id: backend_wx.py 4289 2007-11-14 19:25:46Z mdboom $'
+cvs_id = '$Id: backend_wx.py 5633 2008-06-22 16:24:02Z jdh2358 $'
 
-import sys, os, os.path, math, StringIO
+
+import sys, os, os.path, math, StringIO, weakref, warnings
 
 # Debugging settings here...
 # Debug level set here. If the debug level is less than 5, information
@@ -154,7 +155,9 @@ from matplotlib._pylab_helpers import Gcf
 from matplotlib.artist import Artist
 from matplotlib.cbook import exception_to_str, is_string_like, is_writable_file_like
 from matplotlib.figure import Figure
+from matplotlib.path import Path
 from matplotlib.text import _process_text_args, Text
+from matplotlib.transforms import Affine2D
 from matplotlib.widgets import SubplotTool
 from matplotlib import rcParams
 
@@ -261,10 +264,14 @@ class RendererWx(RendererBase):
         #return 1, 1
         if ismath: s = self.strip_math(s)
 
-        if self.gc is None: gc = self.new_gc()
+        if self.gc is None:
+            gc = self.new_gc()
+        else:
+            gc = self.gc
+        gfx_ctx = gc.gfx_ctx
         font = self.get_wx_font(s, prop)
-        self.gc.SetFont(font)
-        w, h, descent, leading = self.gc.GetFullTextExtent(s)
+        gfx_ctx.SetFont(font, wx.BLACK)
+        w, h, descent, leading = gfx_ctx.GetFullTextExtent(s)
 
         return w, h, descent
 
@@ -272,95 +279,48 @@ class RendererWx(RendererBase):
         'return the canvas width and height in display coords'
         return self.width, self.height
 
+    def handle_clip_rectangle(self, gc):
+        new_bounds = gc.get_clip_rectangle()
+        if new_bounds is not None:
+            new_bounds = new_bounds.bounds
+        gfx_ctx = gc.gfx_ctx
+        if gfx_ctx._lastcliprect != new_bounds:
+            gfx_ctx._lastcliprect = new_bounds
+            if new_bounds is None:
+                gfx_ctx.ResetClip()
+            else:
+                gfx_ctx.Clip(new_bounds[0], self.height - new_bounds[1] - new_bounds[3],
+                             new_bounds[2], new_bounds[3])
 
-    def draw_arc(self, gc, rgbFace, x, y, width, height, angle1, angle2, rotation):
-        """
-        Draw an arc centered at x,y with width and height and angles
-        from 0.0 to 360.0.
-        If rgbFace is present, fill the figure in this colour, otherwise
-        it is not filled.
-        """
+    #@staticmethod
+    def convert_path(gfx_ctx, tpath):
+        wxpath = gfx_ctx.CreatePath()
+        for points, code in tpath.iter_segments():
+            if code == Path.MOVETO:
+                wxpath.MoveToPoint(*points)
+            elif code == Path.LINETO:
+                wxpath.AddLineToPoint(*points)
+            elif code == Path.CURVE3:
+                wxpath.AddQuadCurveToPoint(*points)
+            elif code == Path.CURVE4:
+                wxpath.AddCurveToPoint(*points)
+            elif code == Path.CLOSEPOLY:
+                wxpath.CloseSubpath()
+        return wxpath
+    convert_path = staticmethod(convert_path)
+
+    def draw_path(self, gc, path, transform, rgbFace=None):
         gc.select()
-        assert gc.Ok(), "wxMemoryDC not OK to use"
-        # wxPython requires upper left corner of bounding rectange for ellipse
-        # Theoretically you don't need the int() below, but it seems to make
-        # rounding of arc centre point more accurate in screen co-ordinates
-        ulX = x - int(width/2)
-        ulY = self.height - int(y + (height/2))
+        self.handle_clip_rectangle(gc)
+        gfx_ctx = gc.gfx_ctx
+        transform = transform + Affine2D().scale(1.0, -1.0).translate(0.0, self.height)
+        tpath = transform.transform_path(path)
+        wxpath = self.convert_path(gfx_ctx, tpath)
         if rgbFace is not None:
-            r,g,b = self._to_wx_rgb(rgbFace)
-            new_brush =wx.Brush(wx.Colour(r,g,b), wx.SOLID)
-            gc.SetBrush(new_brush)
+            gfx_ctx.SetBrush(wx.Brush(gc.get_wxcolour(rgbFace)))
+            gfx_ctx.DrawPath(wxpath)
         else:
-            gc.SetBrush(wx.TRANSPARENT_BRUSH)
-        gc.DrawEllipticArc(int(ulX), int(ulY), int(width)+1, int(height)+1,
-                           int(angle1), int(angle2))
-        gc.unselect()
-
-    def draw_line(self, gc, x1, y1, x2, y2):
-        """
-        Draw a single line from x1,y1 to x2,y2
-        """
-        DEBUG_MSG("draw_line()", 1, self)
-        gc.select()
-        gc.DrawLine(int(x1), self.height - int(y1),
-                    int(x2), self.height - int(y2))
-        gc.unselect()
-
-    def draw_lines(self, gc, x, y):
-        """
-        x and y are equal length arrays, draw lines connecting each
-        point in x, y
-        """
-        gc.select()
-        assert gc.Ok(), "wxMemoryDC not OK to use"
-        assert len(x) == len(y), "draw_lines() x and y must be of equal length"
-        gc.DrawLines([wx.Point(int(x[i]), self.height - int(y[i])) for i in range(len(x))])
-        gc.unselect()
-
-    def draw_polygon(self, gc, rgbFace, points):
-        """
-        Draw a polygon.  points is a len vertices tuple, each element
-        giving the x,y coords a vertex
-        """
-        gc.select()
-        assert gc.Ok(), "wxMemoryDC not OK to use"
-        points = [(int(x), self.height - int(y)) for x,y in points]
-        if rgbFace is not None:
-            r,g,b = self._to_wx_rgb(rgbFace)
-            new_brush =wx.Brush(wx.Colour(r,g,b), wx.SOLID)
-            gc.SetBrush(new_brush)
-        else:
-            gc.SetBrush(wx.TRANSPARENT_BRUSH)
-        gc.DrawPolygon(points)
-        gc.unselect()
-
-    def draw_rectangle(self, gc, rgbFace, x, y, width, height):
-        """
-        Draw a rectangle at lower left x,y with width and height
-        If filled=True, fill the rectangle with the gc foreground
-        gc is a GraphicsContext instance
-        """
-        # wxPython uses rectangle from TOP left!
-        gc.select()
-        assert gc.Ok(), "wxMemoryDC not OK to use"
-        if rgbFace is not None:
-            r,g,b = self._to_wx_rgb(rgbFace)
-            new_brush =wx.Brush(wx.Colour(r,g,b), wx.SOLID)
-            gc.SetBrush(new_brush)
-        else:
-            gc.SetBrush(wx.TRANSPARENT_BRUSH)
-        gc.DrawRectangle(int(x), self.height - int(height + y),
-                                 int(math.ceil(width)), int(math.ceil(height)))
-        gc.unselect()
-
-    def draw_point(self, gc, x, y):
-        """
-        Draw a single point at x,y
-        """
-        gc.select()
-        assert gc.Ok(), "wxMemoryDC not OK to use"
-        gc.DrawPoint(int(x), self.height - int(y))
+            gfx_ctx.StrokePath(wxpath)
         gc.unselect()
 
     def draw_text(self, gc, x, y, s, prop, angle, ismath):
@@ -371,26 +331,25 @@ class RendererWx(RendererBase):
         if ismath: s = self.strip_math(s)
         DEBUG_MSG("draw_text()", 1, self)
         gc.select()
+        self.handle_clip_rectangle(gc)
+        gfx_ctx = gc.gfx_ctx
 
         font = self.get_wx_font(s, prop)
-        gc.SetFont(font)
-        assert gc.Ok(), "wxMemoryDC not OK to use"
+        color = gc.get_wxcolour(gc.get_rgb())
+        gfx_ctx.SetFont(font, color)
 
         w, h, d = self.get_text_width_height_descent(s, prop, ismath)
         x = int(x)
         y = int(y-h)
 
-        if angle!=0:
-            # Correct for the fact that text if rotated around the upper-left corner,
-            # rather than the lower-left corner as we expect.
+        if angle == 0.0:
+            gfx_ctx.DrawText(s, x, y)
+        else:
             rads = angle / 180.0 * math.pi
             xo = h * math.sin(rads)
             yo = h * math.cos(rads)
-            try: gc.DrawRotatedText(s, x - xo, y - yo, angle)
-            except:
-                verbose.print_error(exception_to_str('WX rotated text failed'))
-        else:
-            gc.DrawText(s, x, y)
+            gfx_ctx.DrawRotatedText(s, x - xo, y - yo, rads)
+
         gc.unselect()
 
     def new_gc(self):
@@ -400,7 +359,6 @@ class RendererWx(RendererBase):
         DEBUG_MSG('new_gc()', 2, self)
         self.gc = GraphicsContextWx(self.bitmap, self)
         self.gc.select()
-        assert self.gc.Ok(), "wxMemoryDC not OK to use"
         self.gc.unselect()
         return self.gc
 
@@ -451,29 +409,22 @@ class RendererWx(RendererBase):
 
         return font
 
-    def _to_wx_rgb(self, rgb):
-        """Takes a colour value and returns a tuple (r,g,b) suitable
-        for instantiating a wx.Colour."""
-        r, g, b = rgb
-        return (int(r * 255), int(g * 255), int(b * 255))
-
-
 
     def points_to_pixels(self, points):
         """
         convert point measures to pixes using dpi and the pixels per
         inch of the display
         """
-        return points*(PIXELS_PER_INCH/72.0*self.dpi.get()/72.0)
+        return points*(PIXELS_PER_INCH/72.0*self.dpi/72.0)
 
-class GraphicsContextWx(GraphicsContextBase, wx.MemoryDC):
+class GraphicsContextWx(GraphicsContextBase):
     """
     The graphics context provides the color, line styles, etc...
 
-    In wxPython this is done by wrapping a wxDC object and forwarding the
-    appropriate calls to it. Notice also that colour and line styles are
-    mapped on the wx.Pen() member of the wxDC. This means that we have some
-    rudimentary pen management here.
+    This class stores a reference to a wxMemoryDC, and a
+    wxGraphicsContext that draws to it.  Creating a wxGraphicsContext
+    seems to be fairly heavy, so these objects are cached based on the
+    bitmap object that is passed in.
 
     The base GraphicsContext stores colors as a RGB tuple on the unit
     interval, eg, (0.5, 0.0, 1.0).  wxPython uses an int interval, but
@@ -492,25 +443,28 @@ class GraphicsContextWx(GraphicsContextBase, wx.MemoryDC):
                   'dashed':    wx.SHORT_DASH,
                   'dashdot':   wx.DOT_DASH,
                   'dotted':    wx.DOT }
-    _lastWxDC = None
+    _cache = weakref.WeakKeyDictionary()
 
     def __init__(self, bitmap, renderer):
         GraphicsContextBase.__init__(self)
-        wx.MemoryDC.__init__(self)
         #assert self.Ok(), "wxMemoryDC not OK to use"
         DEBUG_MSG("__init__()", 1, self)
-        # Make sure (belt and braces!) that existing wxDC is not selected to
-        # to a bitmap.
-        if GraphicsContextWx._lastWxDC != None:
 
-            GraphicsContextWx._lastWxDC.SelectObject(wx.NullBitmap)
+        dc, gfx_ctx = self._cache.get(bitmap, (None, None))
+        if dc is None:
+            dc = wx.MemoryDC()
+            dc.SelectObject(bitmap)
+            gfx_ctx = wx.GraphicsContext.Create(dc)
+            gfx_ctx._lastcliprect = None
+            self._cache[bitmap] = dc, gfx_ctx
 
-        self.SelectObject(bitmap)
         self.bitmap = bitmap
-        self.SetPen(wx.Pen('BLACK', 1, wx.SOLID))
-        self._style=wx.SOLID
+        self.dc = dc
+        self.gfx_ctx = gfx_ctx
+        self._pen = wx.Pen('BLACK', 1, wx.SOLID)
+        gfx_ctx.SetPen(self._pen)
+        self._style = wx.SOLID
         self.renderer = renderer
-        GraphicsContextWx._lastWxDC = self
 
     def select(self):
         """
@@ -529,23 +483,6 @@ class GraphicsContextWx(GraphicsContextBase, wx.MemoryDC):
             self.SelectObject(wx.NullBitmap)
             self.IsSelected = False
 
-    def set_clip_rectangle(self, rect):
-        """
-        Destroys previous clipping region and defines a new one.
-        """
-        DEBUG_MSG("set_clip_rectangle()", 1, self)
-        self.select()
-        l,b,w,h = rect
-        # this appears to be version dependent'
-        if hasattr(self, 'SetClippingRegionXY'):
-            clipfunc = getattr(self, 'SetClippingRegionXY')
-        else:
-            clipfunc = getattr(self, 'SetClippingRegion')
-
-        clipfunc(int(l), self.renderer.height - int(b+h),
-                 int(w), int(h))
-        self.unselect()
-
     def set_foreground(self, fg, isRGB=None):
         """
         Set the foreground color.  fg can be a matlab format string, a
@@ -561,12 +498,8 @@ class GraphicsContextWx(GraphicsContextBase, wx.MemoryDC):
         self.select()
         GraphicsContextBase.set_foreground(self, fg, isRGB)
 
-        pen = self.GetPen()
-        pen.SetColour(self.get_wxcolour())
-        self.SetPen(pen)
-        brush =wx.Brush(self.get_wxcolour(), wx.SOLID)
-        self.SetBrush(brush)
-        self.SetTextForeground(self.get_wxcolour())
+        self._pen.SetColour(self.get_wxcolour(self.get_rgb()))
+        self.gfx_ctx.SetPen(self._pen)
         self.unselect()
 
     def set_graylevel(self, frac):
@@ -578,11 +511,8 @@ class GraphicsContextWx(GraphicsContextBase, wx.MemoryDC):
         DEBUG_MSG("set_graylevel()", 1, self)
         self.select()
         GraphicsContextBase.set_graylevel(self, frac)
-        pen = self.GetPen()
-        pen.SetColour(self.get_wxcolour())
-        self.SetPen(pen)
-        brush =wx.Brush(self.get_wxcolour(), wx.SOLID)
-        self.SetBrush(brush)
+        self._pen.SetColour(self.get_wxcolour(self.get_rgb()))
+        self.gfx_ctx.SetPen(self._pen)
         self.unselect()
 
     def set_linewidth(self, w):
@@ -593,11 +523,10 @@ class GraphicsContextWx(GraphicsContextBase, wx.MemoryDC):
         self.select()
         if w>0 and w<1: w = 1
         GraphicsContextBase.set_linewidth(self, w)
-        pen = self.GetPen()
         lw = int(self.renderer.points_to_pixels(self._linewidth))
         if lw==0: lw = 1
-        pen.SetWidth(lw)
-        self.SetPen(pen)
+        self._pen.SetWidth(lw)
+        self.gfx_ctx.SetPen(self._pen)
         self.unselect()
 
     def set_capstyle(self, cs):
@@ -607,9 +536,8 @@ class GraphicsContextWx(GraphicsContextBase, wx.MemoryDC):
         DEBUG_MSG("set_capstyle()", 1, self)
         self.select()
         GraphicsContextBase.set_capstyle(self, cs)
-        pen = self.GetPen()
-        pen.SetCap(GraphicsContextWx._capd[self._capstyle])
-        self.SetPen(pen)
+        self._pen.SetCap(GraphicsContextWx._capd[self._capstyle])
+        self.gfx_ctx.SetPen(self._pen)
         self.unselect()
 
     def set_joinstyle(self, js):
@@ -619,9 +547,8 @@ class GraphicsContextWx(GraphicsContextBase, wx.MemoryDC):
         DEBUG_MSG("set_joinstyle()", 1, self)
         self.select()
         GraphicsContextBase.set_joinstyle(self, js)
-        pen = self.GetPen()
-        pen.SetJoin(GraphicsContextWx._joind[self._joinstyle])
-        self.SetPen(pen)
+        self._pen.SetJoin(GraphicsContextWx._joind[self._joinstyle])
+        self.gfx_ctx.SetPen(self._pen)
         self.unselect()
 
     def set_linestyle(self, ls):
@@ -634,25 +561,32 @@ class GraphicsContextWx(GraphicsContextBase, wx.MemoryDC):
         try:
             self._style = GraphicsContextWx._dashd_wx[ls]
         except KeyError:
-            self._style=wx.LONG_DASH# Style not used elsewhere...
+            self._style = wx.LONG_DASH# Style not used elsewhere...
 
         # On MS Windows platform, only line width of 1 allowed for dash lines
         if wx.Platform == '__WXMSW__':
             self.set_linewidth(1)
 
-        pen = self.GetPen()
-        pen.SetStyle(self._style)
-        self.SetPen(pen)
+        self._pen.SetStyle(self._style)
+        self.gfx_ctx.SetPen(self._pen)
         self.unselect()
 
-    def get_wxcolour(self):
+    def get_wxcolour(self, color):
         """return a wx.Colour from RGB format"""
         DEBUG_MSG("get_wx_color()", 1, self)
-        r, g, b = self.get_rgb()
-        r *= 255
-        g *= 255
-        b *= 255
-        return wx.Colour(red=int(r), green=int(g), blue=int(b))
+        if len(color) == 3:
+            r, g, b = color
+            r *= 255
+            g *= 255
+            b *= 255
+            return wx.Colour(red=int(r), green=int(g), blue=int(b))
+        else:
+            r, g, b, a = color
+            r *= 255
+            g *= 255
+            b *= 255
+            a *= 255
+            return wx.Colour(red=int(r), green=int(g), blue=int(b), alpha=int(a))
 
 class FigureCanvasWx(FigureCanvasBase, wx.Panel):
     """
@@ -740,11 +674,22 @@ class FigureCanvasWx(FigureCanvasBase, wx.Panel):
         FigureCanvasBase.__init__(self, figure)
         # Set preferred window size hint - helps the sizer (if one is
         # connected)
-        l,b,w,h = figure.bbox.get_bounds()
+        l,b,w,h = figure.bbox.bounds
         w = int(math.ceil(w))
         h = int(math.ceil(h))
 
         wx.Panel.__init__(self, parent, id, size=wx.Size(w, h))
+
+        def do_nothing(*args, **kwargs):
+            warnings.warn('could not find a setinitialsize function for backend_wx; please report your wxpython version=%s to the matplotlib developers list'%backend_version)
+            pass
+
+        # try to find the set size func across wx versions
+        try:
+            getattr(self, 'SetInitialSize')
+        except AttributeError:
+            self.SetInitialSize = getattr(self, 'SetBestFittingSize', do_nothing)
+
         # Create the drawing bitmap
         self.bitmap =wx.EmptyBitmap(w, h)
         DEBUG_MSG("__init__() - bitmap w:%d h:%d" % (w,h), 2, self)
@@ -943,6 +888,9 @@ The current aspect ration will be kept."""
         self.gui_repaint()
 
 
+    def draw_idle(self, *args, **kwargs):
+        pass
+
     def draw(self, repaint=True):
         """
         Render the figure using RendererWx instance renderer, or using a
@@ -954,6 +902,9 @@ The current aspect ration will be kept."""
         if repaint:
             self.gui_repaint()
 
+    def flush_events(self):
+        wx.Yield()
+
     def _get_imagesave_wildcards(self):
         'return the wildcard string for the filesave dialog'
         default_filetype = self.get_default_filetype()
@@ -986,6 +937,7 @@ The current aspect ration will be kept."""
         drawDC.BeginDrawing()
         drawDC.DrawBitmap(self.bitmap, 0, 0)
         drawDC.EndDrawing()
+        #wx.GetApp().Yield()
 
     filetypes = FigureCanvasBase.filetypes.copy()
     filetypes['bmp'] = 'Windows bitmap'
@@ -999,7 +951,7 @@ The current aspect ration will be kept."""
 
     def print_bmp(self, filename, *args, **kwargs):
         return self._print_image(filename, wx.BITMAP_TYPE_BMP, *args, **kwargs)
-    
+
     def print_jpeg(self, filename, *args, **kwargs):
         return self._print_image(filename, wx.BITMAP_TYPE_JPEG, *args, **kwargs)
     print_jpg = print_jpeg
@@ -1009,18 +961,18 @@ The current aspect ration will be kept."""
 
     def print_png(self, filename, *args, **kwargs):
         return self._print_image(filename, wx.BITMAP_TYPE_PNG, *args, **kwargs)
-    
+
     def print_tiff(self, filename, *args, **kwargs):
         return self._print_image(filename, wx.BITMAP_TYPE_TIF, *args, **kwargs)
     print_tif = print_tiff
 
     def print_xpm(self, filename, *args, **kwargs):
         return self._print_image(filename, wx.BITMAP_TYPE_XPM, *args, **kwargs)
-    
+
     def _print_image(self, filename, filetype, *args, **kwargs):
         origBitmap   = self.bitmap
 
-        l,b,width,height = self.figure.bbox.get_bounds()
+        l,b,width,height = self.figure.bbox.bounds
         width = int(math.ceil(width))
         height = int(math.ceil(height))
 
@@ -1055,7 +1007,7 @@ The current aspect ration will be kept."""
 
     def get_default_filetype(self):
         return 'png'
-        
+
     def realize(self):
         """
         This method will be called when the system is ready to draw,
@@ -1102,7 +1054,7 @@ The current aspect ration will be kept."""
         if not self._isConfigured:
             self._isConfigured = True
 
-        dpival = self.figure.dpi.get()
+        dpival = self.figure.dpi
         winch = self._width/dpival
         hinch = self._height/dpival
         self.figure.set_size_inches(winch, hinch)
@@ -1142,7 +1094,7 @@ The current aspect ration will be kept."""
     def _onRightButtonDown(self, evt):
         """Start measuring on an axis."""
         x = evt.GetX()
-        y = self.figure.bbox.height() - evt.GetY()
+        y = self.figure.bbox.height - evt.GetY()
         evt.Skip()
         self.CaptureMouse()
         FigureCanvasBase.button_press_event(self, x, y, 3, guiEvent=evt)
@@ -1151,7 +1103,7 @@ The current aspect ration will be kept."""
     def _onRightButtonUp(self, evt):
         """End measuring on an axis."""
         x = evt.GetX()
-        y = self.figure.bbox.height() - evt.GetY()
+        y = self.figure.bbox.height - evt.GetY()
         evt.Skip()
         if self.HasCapture(): self.ReleaseMouse()
         FigureCanvasBase.button_release_event(self, x, y, 3, guiEvent=evt)
@@ -1159,7 +1111,7 @@ The current aspect ration will be kept."""
     def _onLeftButtonDown(self, evt):
         """Start measuring on an axis."""
         x = evt.GetX()
-        y = self.figure.bbox.height() - evt.GetY()
+        y = self.figure.bbox.height - evt.GetY()
         evt.Skip()
         self.CaptureMouse()
         FigureCanvasBase.button_press_event(self, x, y, 1, guiEvent=evt)
@@ -1167,7 +1119,7 @@ The current aspect ration will be kept."""
     def _onLeftButtonUp(self, evt):
         """End measuring on an axis."""
         x = evt.GetX()
-        y = self.figure.bbox.height() - evt.GetY()
+        y = self.figure.bbox.height - evt.GetY()
         #print 'release button', 1
         evt.Skip()
         if self.HasCapture(): self.ReleaseMouse()
@@ -1181,7 +1133,7 @@ The current aspect ration will be kept."""
         """Start measuring on an axis."""
 
         x = evt.GetX()
-        y = self.figure.bbox.height() - evt.GetY()
+        y = self.figure.bbox.height - evt.GetY()
         evt.Skip()
         FigureCanvasBase.motion_notify_event(self, x, y, guiEvent=evt)
 
@@ -1284,16 +1236,17 @@ class FigureFrameWx(wx.Frame):
             pos = wx.DefaultPosition
         else:
             pos =wx.Point(20,20)
-        l,b,w,h = fig.bbox.get_bounds()
+        l,b,w,h = fig.bbox.bounds
         wx.Frame.__init__(self, parent=None, id=-1, pos=pos,
-                          title="Figure %d" % num,
-                          size=(w,h))
+                          title="Figure %d" % num)
+        # Frame will be sized later by the Fit method
         DEBUG_MSG("__init__()", 1, self)
         self.num = num
 
-        self.canvas = self.get_canvas(fig)
         statbar = StatusBarWx(self)
         self.SetStatusBar(statbar)
+        self.canvas = self.get_canvas(fig)
+        self.canvas.SetInitialSize(wx.Size(fig.bbox.width, fig.bbox.height))
         self.sizer =wx.BoxSizer(wx.VERTICAL)
         self.sizer.Add(self.canvas, 1, wx.TOP | wx.LEFT | wx.EXPAND)
         # By adding toolbar in sizer, we are able to put it at the bottom
@@ -1393,9 +1346,9 @@ class FigureManagerWx(FigureManagerBase):
         self.canvas.figure.add_axobserver(notify_axes_change)
 
         def showfig(*args):
-            figwin.frame.Show()
-            figwin.canvas.realize()
-            figwin.canvas.draw()
+            frame.Show()
+            canvas.realize()
+            canvas.draw()
 
         # attach a show method to the figure
         self.canvas.figure.show = showfig
@@ -1412,6 +1365,11 @@ class FigureManagerWx(FigureManagerBase):
     def set_window_title(self, title):
         self.window.SetTitle(title)
 
+    def resize(self, width, height):
+        'Set the canvas size in pixels'
+        self.canvas.SetInitialSize(wx.Size(width, height))
+        self.window.GetSizer().Fit(self.window)
+
 # Identifiers for toolbar controls - images_wx contains bitmaps for the images
 # used in the controls. wxWindows does not provide any stock images, so I've
 # 'stolen' those from GTK2, and transformed them into the appropriate format.
@@ -1437,9 +1395,6 @@ def _load_bitmap(filename):
     matplotlib library is installed. The filename parameter should not
     contain any path information as this is determined automatically.
 
-    Bitmaps should be in XPM format, and of size 16x16 (unless you change
-    the code!). I have converted the stock GTK2 16x16 icons to XPM format.
-
     Returns a wx.Bitmap object
     """
 
@@ -1449,7 +1404,7 @@ def _load_bitmap(filename):
     if not os.path.exists(bmpFilename):
         raise IOError('Could not find bitmap file "%s"; dying'%bmpFilename)
 
-    bmp =wx.Bitmap(bmpFilename, wx.BITMAP_TYPE_XPM)
+    bmp = wx.Bitmap(bmpFilename)
     return bmp
 
 class MenuButtonWx(wx.Button):
@@ -1612,24 +1567,24 @@ class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
 
         self.SetToolBitmapSize(wx.Size(24,24))
 
-        self.AddSimpleTool(_NTB2_HOME, _load_bitmap('home.xpm'),
+        self.AddSimpleTool(_NTB2_HOME, _load_bitmap('home.png'),
                            'Home', 'Reset original view')
-        self.AddSimpleTool(self._NTB2_BACK, _load_bitmap('back.xpm'),
+        self.AddSimpleTool(self._NTB2_BACK, _load_bitmap('back.png'),
                            'Back', 'Back navigation view')
-        self.AddSimpleTool(self._NTB2_FORWARD, _load_bitmap('forward.xpm'),
+        self.AddSimpleTool(self._NTB2_FORWARD, _load_bitmap('forward.png'),
                            'Forward', 'Forward navigation view')
         # todo: get new bitmap
-        self.AddCheckTool(self._NTB2_PAN, _load_bitmap('move.xpm'),
+        self.AddCheckTool(self._NTB2_PAN, _load_bitmap('move.png'),
                            shortHelp='Pan',
                            longHelp='Pan with left, zoom with right')
-        self.AddCheckTool(self._NTB2_ZOOM, _load_bitmap('zoom_to_rect.xpm'),
+        self.AddCheckTool(self._NTB2_ZOOM, _load_bitmap('zoom_to_rect.png'),
                            shortHelp='Zoom', longHelp='Zoom to rectangle')
 
         self.AddSeparator()
-        self.AddSimpleTool(_NTB2_SUBPLOT, _load_bitmap('subplots.xpm'),
+        self.AddSimpleTool(_NTB2_SUBPLOT, _load_bitmap('subplots.png'),
                            'Configure subplots', 'Configure subplot parameters')
 
-        self.AddSimpleTool(_NTB2_SAVE, _load_bitmap('filesave.xpm'),
+        self.AddSimpleTool(_NTB2_SAVE, _load_bitmap('filesave.png'),
                            'Save', 'Save plot contents to file')
 
         if wx.VERSION_STRING >= '2.5':
@@ -1692,8 +1647,14 @@ class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
             filename = dlg.GetFilename()
             DEBUG_MSG('Save file dir:%s name:%s' % (dirname, filename), 3, self)
             format = exts[dlg.GetFilterIndex()]
-            # Explicitly pass in the selected filetype to override the
-            # actual extension if necessary
+            basename, ext = os.path.splitext(filename)
+            if ext.startswith('.'):
+                ext = ext[1:]
+            if ext in ('svg', 'pdf', 'ps', 'eps', 'png') and format!=ext:
+                #looks like they forgot to set the image type drop
+                #down, going with the extension.
+                warnings.warn('extension %s did not match the selected image type %s; going with %s'%(ext, format, ext), stacklevel=0)
+                format = ext
             try:
                 self.canvas.print_figure(
                     os.path.join(dirname, filename), format=format)
@@ -1740,7 +1701,7 @@ class NavigationToolbar2Wx(NavigationToolbar2, wx.ToolBar):
 
         dc.ResetBoundingBox()
         dc.BeginDrawing()
-        height = self.canvas.figure.bbox.height()
+        height = self.canvas.figure.bbox.height
         y1 = height - y1
         y0 = height - y0
 
@@ -1890,28 +1851,28 @@ class NavigationToolbarWx(wx.ToolBar):
 
         DEBUG_MSG("panx()", 1, self)
         for a in self._active:
-            a.panx(direction)
+            a.xaxis.pan(direction)
         self.canvas.draw()
         self.canvas.Refresh(eraseBackground=False)
 
     def pany(self, direction):
         DEBUG_MSG("pany()", 1, self)
         for a in self._active:
-            a.pany(direction)
+            a.yaxis.pan(direction)
         self.canvas.draw()
         self.canvas.Refresh(eraseBackground=False)
 
     def zoomx(self, in_out):
         DEBUG_MSG("zoomx()", 1, self)
         for a in self._active:
-            a.zoomx(in_out)
+            a.xaxis.zoom(in_out)
         self.canvas.draw()
         self.canvas.Refresh(eraseBackground=False)
 
     def zoomy(self, in_out):
         DEBUG_MSG("zoomy()", 1, self)
         for a in self._active:
-            a.zoomy(in_out)
+            a.yaxis.zoom(in_out)
         self.canvas.draw()
         self.canvas.Refresh(eraseBackground=False)
 
@@ -2046,13 +2007,13 @@ class PrintoutWx(wx.Printout):
         # so that we can temporarily set them to the dpi of
         # the printer, and the bg color to white
         bgcolor   = self.canvas.figure.get_facecolor()
-        fig_dpi   = self.canvas.figure.dpi.get()
+        fig_dpi   = self.canvas.figure.dpi
 
         # draw the bitmap, scaled appropriately
         vscale    = float(ppw) / fig_dpi
 
         # set figure resolution,bg color for printer
-        self.canvas.figure.dpi.set(ppw)
+        self.canvas.figure.dpi = ppw
         self.canvas.figure.set_facecolor('#FFFFFF')
 
         renderer  = RendererWx(self.canvas.bitmap, self.canvas.figure.dpi)
