@@ -5,11 +5,7 @@ from the Python Cookbook -- hence the name cbook
 from __future__ import generators
 import re, os, errno, sys, StringIO, traceback, locale
 import time, datetime
-import numpy as npy
-
-try: set
-except NameError:
-    from sets import Set as set
+import numpy as np
 
 major, minor1, minor2, s, tmp = sys.version_info
 
@@ -44,7 +40,7 @@ class tostr(converter):
 class todatetime(converter):
     'convert to a datetime or None'
     def __init__(self, fmt='%Y-%m-%d', missing='Null', missingval=None):
-        'use a time.strptime format string for conversion'
+        'use a :func:`time.strptime` format string for conversion'
         converter.__init__(self, missing, missingval)
         self.fmt = fmt
 
@@ -58,7 +54,7 @@ class todatetime(converter):
 class todate(converter):
     'convert to a date or None'
     def __init__(self, fmt='%Y-%m-%d', missing='Null', missingval=None):
-        'use a time.strptime format string for conversion'
+        'use a :func:`time.strptime` format string for conversion'
         converter.__init__(self, missing, missingval)
         self.fmt = fmt
     def __call__(self, s):
@@ -88,7 +84,7 @@ class toint(converter):
 class CallbackRegistry:
     """
     Handle registering and disconnecting for a set of signals and
-    callbacks
+    callbacks::
 
        signals = 'eat', 'drink', 'be merry'
 
@@ -113,7 +109,7 @@ class CallbackRegistry:
 
     """
     def __init__(self, signals):
-        'signals is a sequence of valid signals'
+        '*signals* is a sequence of valid signals'
         self.signals = set(signals)
         # callbacks is a dict mapping the signal to a dictionary
         # mapping callback id to the callback function
@@ -121,7 +117,7 @@ class CallbackRegistry:
         self._cid = 0
 
     def _check_signal(self, s):
-        'make sure s is a valid signal or raise a ValueError'
+        'make sure *s* is a valid signal or raise a ValueError'
         if s not in self.signals:
             signals = list(self.signals)
             signals.sort()
@@ -129,7 +125,7 @@ class CallbackRegistry:
 
     def connect(self, s, func):
         """
-        register func to be called when a signal s is generated
+        register *func* to be called when a signal *s* is generated
         func will be called
         """
         self._check_signal(s)
@@ -139,7 +135,7 @@ class CallbackRegistry:
 
     def disconnect(self, cid):
         """
-        disconnect the callback registered with callback id cid
+        disconnect the callback registered with callback id *cid*
         """
         for eventname, callbackd in self.callbacks.items():
             try: del callbackd[cid]
@@ -148,8 +144,8 @@ class CallbackRegistry:
 
     def process(self, s, *args, **kwargs):
         """
-        process signal s.  All of the functions registered to receive
-        callbacks on s will be called with *args and **kwargs
+        process signal *s*.  All of the functions registered to receive
+        callbacks on *s* will be called with *\*args* and *\*\*kwargs*
         """
         self._check_signal(s)
         for func in self.callbacks[s].values():
@@ -175,60 +171,65 @@ class silent_list(list):
 
 def strip_math(s):
     'remove latex formatting from mathtext'
-    remove = (r'\rm', '\cal', '\tt', '\it', '\\', '{', '}')
+    remove = (r'\mathdefault', r'\rm', r'\cal', r'\tt', r'\it', '\\', '{', '}')
     s = s[1:-1]
     for r in remove:  s = s.replace(r,'')
     return s
 
 class Bunch:
-   """
-   Often we want to just collect a bunch of stuff together, naming each
-   item of the bunch; a dictionary's OK for that, but a small do- nothing
-   class is even handier, and prettier to use.  Whenever you want to
-   group a few variables:
+    """
+    Often we want to just collect a bunch of stuff together, naming each
+    item of the bunch; a dictionary's OK for that, but a small do- nothing
+    class is even handier, and prettier to use.  Whenever you want to
+    group a few variables:
 
-     >>> point = Bunch(datum=2, squared=4, coord=12)
-     >>> point.datum
+      >>> point = Bunch(datum=2, squared=4, coord=12)
+      >>> point.datum
 
-     By: Alex Martelli
-     From: http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/52308
-   """
-   def __init__(self, **kwds):
-      self.__dict__.update(kwds)
+      By: Alex Martelli
+      From: http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/52308
+    """
+    def __init__(self, **kwds):
+        self.__dict__.update(kwds)
 
 
 def unique(x):
-   'Return a list of unique elements of x'
-   return dict([ (val, 1) for val in x]).keys()
+    'Return a list of unique elements of *x*'
+    return dict([ (val, 1) for val in x]).keys()
 
 def iterable(obj):
+    'return true if *obj* is iterable'
     try: len(obj)
     except: return 0
     return 1
 
 
 def is_string_like(obj):
+    'return true if *obj* looks like a string'
     if hasattr(obj, 'shape'): return 0
     try: obj + ''
     except (TypeError, ValueError): return 0
     return 1
 
 def is_writable_file_like(obj):
+    'return true if *obj* looks like a file object with a *write* method'
     return hasattr(obj, 'write') and callable(obj.write)
 
 def is_scalar(obj):
+    'return true if *obj* is not string like and is not iterable'
     return is_string_like(obj) or not iterable(obj)
 
 def is_numlike(obj):
+    'return true if *obj* looks like a number'
     try: obj+1
     except TypeError: return False
     else: return True
 
 def to_filehandle(fname, flag='r', return_opened=False):
     """
-    fname can be a filename or a file handle.  Support for gzipped
-    files is automatic, if the filename ends in .gz.  flag is a
-    read/write flag for file
+    *fname* can be a filename or a file handle.  Support for gzipped
+    files is automatic, if the filename ends in .gz.  *flag* is a
+    read/write flag for :func:`file`
     """
     if is_string_like(fname):
         if fname.endswith('.gz'):
@@ -265,55 +266,56 @@ def flatten(seq, scalarp=is_scalar):
         if scalarp(item): yield item
         else:
             for subitem in flatten(item, scalarp):
-               yield subitem
+                yield subitem
 
 
 
 class Sorter:
-   """
+    """
 
-   Sort by attribute or item
+    Sort by attribute or item
 
-   Example usage:
-   sort = Sorter()
+    Example usage::
 
-   list = [(1, 2), (4, 8), (0, 3)]
-   dict = [{'a': 3, 'b': 4}, {'a': 5, 'b': 2}, {'a': 0, 'b': 0},
-   {'a': 9, 'b': 9}]
+      sort = Sorter()
+
+      list = [(1, 2), (4, 8), (0, 3)]
+      dict = [{'a': 3, 'b': 4}, {'a': 5, 'b': 2}, {'a': 0, 'b': 0},
+              {'a': 9, 'b': 9}]
 
 
-   sort(list)       # default sort
-   sort(list, 1)    # sort by index 1
-   sort(dict, 'a')  # sort a list of dicts by key 'a'
+      sort(list)       # default sort
+      sort(list, 1)    # sort by index 1
+      sort(dict, 'a')  # sort a list of dicts by key 'a'
 
-   """
+    """
 
-   def _helper(self, data, aux, inplace):
-      aux.sort()
-      result = [data[i] for junk, i in aux]
-      if inplace: data[:] = result
-      return result
+    def _helper(self, data, aux, inplace):
+        aux.sort()
+        result = [data[i] for junk, i in aux]
+        if inplace: data[:] = result
+        return result
 
-   def byItem(self, data, itemindex=None, inplace=1):
-      if itemindex is None:
-         if inplace:
-            data.sort()
-            result = data
-         else:
-            result = data[:]
-            result.sort()
-         return result
-      else:
-         aux = [(data[i][itemindex], i) for i in range(len(data))]
-         return self._helper(data, aux, inplace)
+    def byItem(self, data, itemindex=None, inplace=1):
+        if itemindex is None:
+            if inplace:
+                data.sort()
+                result = data
+            else:
+                result = data[:]
+                result.sort()
+            return result
+        else:
+            aux = [(data[i][itemindex], i) for i in range(len(data))]
+            return self._helper(data, aux, inplace)
 
-   def byAttribute(self, data, attributename, inplace=1):
-      aux = [(getattr(data[i],attributename),i) for i in range(len(data))]
-      return self._helper(data, aux, inplace)
+    def byAttribute(self, data, attributename, inplace=1):
+        aux = [(getattr(data[i],attributename),i) for i in range(len(data))]
+        return self._helper(data, aux, inplace)
 
-   # a couple of handy synonyms
-   sort = byItem
-   __call__ = byItem
+    # a couple of handy synonyms
+    sort = byItem
+    __call__ = byItem
 
 
 
@@ -323,19 +325,19 @@ class Xlator(dict):
     """
     All-in-one multiple-string-substitution class
 
-    Example usage:
+    Example usage::
 
-    text = "Larry Wall is the creator of Perl"
-    adict = {
-    "Larry Wall" : "Guido van Rossum",
-    "creator" : "Benevolent Dictator for Life",
-    "Perl" : "Python",
-    }
+      text = "Larry Wall is the creator of Perl"
+      adict = {
+      "Larry Wall" : "Guido van Rossum",
+      "creator" : "Benevolent Dictator for Life",
+      "Perl" : "Python",
+      }
 
-    print multiple_replace(adict, text)
+      print multiple_replace(adict, text)
 
-    xlat = Xlator(adict)
-    print xlat.xlat(text)
+      xlat = Xlator(adict)
+      print xlat.xlat(text)
     """
 
     def _make_regex(self):
@@ -343,11 +345,11 @@ class Xlator(dict):
         return re.compile("|".join(map(re.escape, self.keys())))
 
     def __call__(self, match):
-        """ Handler invoked for each regex match """
+        """ Handler invoked for each regex *match* """
         return self[match.group(0)]
 
     def xlat(self, text):
-        """ Translate text, returns the modified text. """
+        """ Translate *text*, returns the modified text. """
         return self._make_regex().sub(self, text)
 
 
@@ -397,11 +399,11 @@ class Null:
 
 
 def mkdirs(newdir, mode=0777):
-   try: os.makedirs(newdir, mode)
-   except OSError, err:
-      # Reraise the error unless it's about an already existing directory
-      if err.errno != errno.EEXIST or not os.path.isdir(newdir):
-         raise
+    try: os.makedirs(newdir, mode)
+    except OSError, err:
+        # Reraise the error unless it's about an already existing directory
+        if err.errno != errno.EEXIST or not os.path.isdir(newdir):
+            raise
 
 
 class GetRealpathAndStat:
@@ -412,15 +414,18 @@ class GetRealpathAndStat:
         result = self._cache.get(path)
         if result is None:
             realpath = os.path.realpath(path)
-            stat = os.stat(realpath)
-            stat_key = (stat.st_ino, stat.st_dev)
+            if sys.platform == 'win32':
+                stat_key = realpath
+            else:
+                stat = os.stat(realpath)
+                stat_key = (stat.st_ino, stat.st_dev)
             result = realpath, stat_key
             self._cache[path] = result
         return result
 get_realpath_and_stat = GetRealpathAndStat()
 
 def dict_delall(d, keys):
-    'delete all of the keys from the dict d'
+    'delete all of the *keys* from the :class:`dict` *d*'
     for key in keys:
         try: del d[key]
         except KeyError: pass
@@ -455,65 +460,28 @@ class RingBuffer:
         return self.data
 
     def __get_item__(self, i):
-       return self.data[i % len(self.data)]
+        return self.data[i % len(self.data)]
 
 
-# use enumerate builtin if available, else use python version
-try:
-    import __builtin__
-    enumerate = __builtin__.enumerate
-except:
-    def enumerate(seq):
-        """Python equivalent to the enumerate builtin function
-        enumerate() is new in Python 2.3
-        """
-        for i in range(len(seq)):
-            yield i, seq[i]
-
-
-# use reversed builtin if available, else use python version
-try:
-    import __builtin__
-    reversed = __builtin__.reversed
-except:
-    def reversed(seq):
-        """Python equivalent to the enumerate builtin function
-        enumerate() is new in Python 2.3
-        """
-        for i in range(len(seq)-1,-1,-1):
-            yield seq[i]
-
-
-# use itertools.izip if available, else use python version
-try:
-    import itertools
-    izip = itertools.izip
-except:
-    def izip(*iterables):
-        """Python equivalent to itertools.izip
-        itertools module - new in Python 2.3
-        """
-        iterables = map(iter, iterables)
-        while iterables:
-            result = [i.next() for i in iterables]
-            yield tuple(result)
-
 
 def get_split_ind(seq, N):
-   """seq is a list of words.  Return the index into seq such that
-   len(' '.join(seq[:ind])<=N
-   """
+    """
+    *seq* is a list of words.  Return the index into seq such that::
 
-   sLen = 0
-   # todo: use Alex's xrange pattern from the cbook for efficiency
-   for (word, ind) in zip(seq, range(len(seq))):
-      sLen += len(word) + 1  # +1 to account for the len(' ')
-      if sLen>=N: return ind
-   return len(seq)
+        len(' '.join(seq[:ind])<=N
+
+    """
+
+    sLen = 0
+    # todo: use Alex's xrange pattern from the cbook for efficiency
+    for (word, ind) in zip(seq, range(len(seq))):
+        sLen += len(word) + 1  # +1 to account for the len(' ')
+        if sLen>=N: return ind
+    return len(seq)
 
 
 def wrap(prefix, text, cols):
-    'wrap text with prefix at length cols'
+    'wrap *text* with *prefix* at length *cols*'
     pad = ' '*len(prefix.expandtabs())
     available = cols - len(pad)
 
@@ -540,14 +508,13 @@ _find_dedent_regex = re.compile("(?:(?:\n\r?)|^)( *)\S")
 _dedent_regex = {}
 def dedent(s):
     """
-    Remove excess indentation from docstrings.
+    Remove excess indentation from docstring *s*.
 
-    Discards any leading blank lines, then removes up to
-    n whitespace characters from each line, where n is
-    the number of leading whitespace characters in the
-    first line. It differs from textwrap.dedent in its
-    deletion of leading blank lines and its use of the
-    first non-blank line to determine the indentation.
+    Discards any leading blank lines, then removes up to n whitespace
+    characters from each line, where n is the number of leading
+    whitespace characters in the first line. It differs from
+    textwrap.dedent in its deletion of leading blank lines and its use
+    of the first non-blank line to determine the indentation.
 
     It is also faster in most cases.
     """
@@ -582,6 +549,7 @@ def dedent(s):
 def listFiles(root, patterns='*', recurse=1, return_folders=0):
     """
     Recursively list files
+
     from Parmar and Martelli in the Python Cookbook
     """
     import os.path, fnmatch
@@ -611,8 +579,8 @@ def listFiles(root, patterns='*', recurse=1, return_folders=0):
 
 def get_recursive_filelist(args):
     """
-    Recurs all the files and dirs in args ignoring symbolic links and
-    return the files as a list of strings
+    Recurs all the files and dirs in *args* ignoring symbolic links
+    and return the files as a list of strings
     """
     files = []
 
@@ -629,26 +597,26 @@ def get_recursive_filelist(args):
 
 
 def pieces(seq, num=2):
-   "Break up the seq into num tuples"
-   start = 0
-   while 1:
-      item = seq[start:start+num]
-      if not len(item): break
-      yield item
-      start += num
+    "Break up the *seq* into *num* tuples"
+    start = 0
+    while 1:
+        item = seq[start:start+num]
+        if not len(item): break
+        yield item
+        start += num
 
 def exception_to_str(s = None):
 
-   sh = StringIO.StringIO()
-   if s is not None: print >>sh, s
-   traceback.print_exc(file=sh)
-   return sh.getvalue()
+    sh = StringIO.StringIO()
+    if s is not None: print >>sh, s
+    traceback.print_exc(file=sh)
+    return sh.getvalue()
 
 
 def allequal(seq):
     """
-    return true if all elements of seq compare equal.  If seq is 0 or
-    1 length, return True
+    Return *True* if all elements of *seq* compare equal.  If *seq* is
+    0 or 1 length, return *True*
     """
     if len(seq)<2: return True
     val = seq[0]
@@ -658,14 +626,20 @@ def allequal(seq):
     return True
 
 def alltrue(seq):
-    #return true if all elements of seq are true.  If seq is empty return false
+    """
+    Return *True* if all elements of *seq* evaluate to *True*.  If
+    *seq* is empty, return *False*.
+    """
     if not len(seq): return False
     for val in seq:
         if not val: return False
     return True
 
 def onetrue(seq):
-    #return true if one element of seq is true.  If seq is empty return false
+    """
+    Return *True* if one element of *seq* is *True*.  It *seq* is
+    empty, return *False*.
+    """
     if not len(seq): return False
     for val in seq:
         if val: return True
@@ -673,10 +647,11 @@ def onetrue(seq):
 
 def allpairs(x):
     """
-    return all possible pairs in sequence x
+    return all possible pairs in sequence *x*
 
-    Condensed by Alex Martelli from this thread on c.l.python
-    http://groups.google.com/groups?q=all+pairs+group:*python*&hl=en&lr=&ie=UTF-8&selm=mailman.4028.1096403649.5135.python-list%40python.org&rnum=1
+    Condensed by Alex Martelli from this thread_ on c.l.python
+
+    .. _thread: http://groups.google.com/groups?q=all+pairs+group:*python*&hl=en&lr=&ie=UTF-8&selm=mailman.4028.1096403649.5135.python-list%40python.org&rnum=1
     """
     return [ (s, f) for i, f in enumerate(x) for s in x[i+1:] ]
 
@@ -686,15 +661,17 @@ def allpairs(x):
 # python 2.2 dicts don't have pop--but we don't support 2.2 any more
 def popd(d, *args):
     """
-    Should behave like python2.3 pop method; d is a dict
+    Should behave like python2.3 :meth:`dict.pop` method; *d* is a
+    :class:`dict`::
 
-    # returns value for key and deletes item; raises a KeyError if key
-    # is not in dict
-    val = popd(d, key)
+      # returns value for key and deletes item; raises a KeyError if key
+      # is not in dict
+      val = popd(d, key)
+
+      # returns value for key if key exists, else default.  Delete key,
+      # val item if it exists.  Will not raise a KeyError
+      val = popd(d, key, default)
 
-    # returns value for key if key exists, else default.  Delete key,
-    # val item if it exists.  Will not raise a KeyError
-    val = popd(d, key, default)
     """
     if len(args)==1:
         key = args[0]
@@ -780,8 +757,8 @@ class Stack:
 
     def bubble(self, o):
         """
-        raise o to the top of the stack and return o.  o must be in
-        the stack
+        raise *o* to the top of the stack and return *o*.  *o* must be
+        in the stack
         """
 
         if o not in self._elements:
@@ -797,7 +774,7 @@ class Stack:
         return o
 
     def remove(self, o):
-        'remove element o from the stack'
+        'remove element *o* from the stack'
         if o not in self._elements:
             raise ValueError('Unknown element o')
         old = self._elements[:]
@@ -812,7 +789,7 @@ def popall(seq):
 
 def finddir(o, match, case=False):
     """
-    return all attributes of o which match string in match.  if case
+    return all attributes of *o* which match string in match.  if case
     is True require an exact case match.
     """
     if case:
@@ -823,7 +800,7 @@ def finddir(o, match, case=False):
     return [orig for name, orig in names if name.find(match)>=0]
 
 def reverse_dict(d):
-    'reverse the dictionary -- may lose data if values are not uniq!'
+    'reverse the dictionary -- may lose data if values are not unique!'
     return dict([(v,k) for k,v in d.items()])
 
 
@@ -844,7 +821,7 @@ def report_memory(i=0):  # argument may go away
 
 _safezip_msg = 'In safezip, len(args[0])=%d but len(args[%d])=%d'
 def safezip(*args):
-    'make sure args are equal len before zipping'
+    'make sure *args* are equal len before zipping'
     Nx = len(args[0])
     for i, arg in enumerate(args[1:]):
         if len(arg) != Nx:
@@ -855,7 +832,7 @@ def safezip(*args):
 class MemoryMonitor:
     def __init__(self, nmax=20000):
         self._nmax = nmax
-        self._mem = npy.zeros((self._nmax,), npy.int32)
+        self._mem = np.zeros((self._nmax,), np.int32)
         self.clear()
 
     def clear(self):
@@ -891,7 +868,7 @@ class MemoryMonitor:
             print "Warning: array size was too small for the number of calls."
 
     def xy(self, i0=0, isub=1):
-        x = npy.arange(i0, self._n, isub)
+        x = np.arange(i0, self._n, isub)
         return x, self._mem[i0:self._n:isub]
 
     def plot(self, i0=0, isub=1, fig=None):
@@ -906,12 +883,16 @@ class MemoryMonitor:
 
 def print_cycles(objects, outstream=sys.stdout, show_progress=False):
     """
-    objects:       A list of objects to find cycles in.  It is often useful
-                   to pass in gc.garbage to find the cycles that are
-                   preventing some objects from being garbage collected.
-    outstream:     The stream for output.
-    show_progress: If True, print the number of objects reached as they are
-                   found.
+    *objects*
+        A list of objects to find cycles in.  It is often useful to
+        pass in gc.garbage to find the cycles that are preventing some
+        objects from being garbage collected.
+
+    *outstream*
+        The stream for output.
+
+    *show_progress*
+        If True, print the number of objects reached as they are found.
     """
     import gc
     from types import FrameType
@@ -966,9 +947,128 @@ def print_cycles(objects, outstream=sys.stdout, show_progress=False):
         outstream.write("Examining: %r\n" % (obj,))
         recurse(obj, obj, { }, [])
 
+class Grouper(object):
+    """
+    This class provides a lightweight way to group arbitrary objects
+    together into disjoint sets when a full-blown graph data structure
+    would be overkill.
+
+    Objects can be joined using :meth:`join`, tested for connectedness
+    using :meth:`joined`, and all disjoint sets can be retreived by
+    using the object as an iterator.
+
+    The objects being joined must be hashable.
+
+    For example:
+
+    >>> g = grouper.Grouper()
+    >>> g.join('a', 'b')
+    >>> g.join('b', 'c')
+    >>> g.join('d', 'e')
+    >>> list(g.get())
+    [['a', 'b', 'c'], ['d', 'e']]
+    >>> g.joined('a', 'b')
+    True
+    >>> g.joined('a', 'c')
+    True
+    >>> g.joined('a', 'd')
+    False
+    """
+    def __init__(self, init=[]):
+        mapping = self._mapping = {}
+        for x in init:
+            mapping[x] = [x]
+
+    def __contains__(self, item):
+        return item in self._mapping
+
+    def join(self, a, *args):
+        """
+        Join given arguments into the same set.  Accepts one or more
+        arguments.
+        """
+        mapping = self._mapping
+        set_a = mapping.setdefault(a, [a])
+
+        for arg in args:
+            set_b = mapping.get(arg)
+            if set_b is None:
+                set_a.append(arg)
+                mapping[arg] = set_a
+            elif set_b is not set_a:
+                if len(set_b) > len(set_a):
+                    set_a, set_b = set_b, set_a
+                set_a.extend(set_b)
+                for elem in set_b:
+                    mapping[elem] = set_a
+
+    def joined(self, a, b):
+        """
+        Returns True if *a* and *b* are members of the same set.
+        """
+        mapping = self._mapping
+        try:
+            return mapping[a] is mapping[b]
+        except KeyError:
+            return False
+
+    def __iter__(self):
+        """
+        Returns an iterator yielding each of the disjoint sets as a list.
+        """
+        seen = set()
+        for elem, group in self._mapping.iteritems():
+            if elem not in seen:
+                yield group
+                seen.update(group)
+
+    def get_siblings(self, a):
+        """
+        Returns all of the items joined with *a*, including itself.
+        """
+        return self._mapping.get(a, [a])
+
+
+def simple_linear_interpolation(a, steps):
+    steps = np.floor(steps)
+    new_length = ((len(a) - 1) * steps) + 1
+    new_shape = list(a.shape)
+    new_shape[0] = new_length
+    result = np.zeros(new_shape, a.dtype)
+
+    result[0] = a[0]
+    a0 = a[0:-1]
+    a1 = a[1:  ]
+    delta = ((a1 - a0) / steps)
+
+    for i in range(1, int(steps)):
+        result[i::steps] = delta * i + a0
+    result[steps::steps] = a1
+
+    return result
+
+def recursive_remove(path):
+    if os.path.isdir(path):
+        for fname in glob.glob(os.path.join(path, '*')) + glob.glob(os.path.join(path, '.*')):
+            if os.path.isdir(fname):
+                recursive_remove(fname)
+                os.removedirs(fname)
+            else:
+                os.remove(fname)
+        #os.removedirs(path)
+    else:
+        os.remove(path)
 
 
 
+# a dict to cross-map linestyle arguments
+_linestyles = [('-', 'solid'),
+    ('--', 'dashed'),
+    ('-.', 'dashdot'),
+    (':',  'dotted')]
+
+ls_mapper = dict(_linestyles)
+ls_mapper.update([(ls[1], ls[0]) for ls in _linestyles])
 
 if __name__=='__main__':
     assert( allequal([1,1,1]) )
