@@ -1,7 +1,7 @@
 from Numeric import array, ravel, reshape, shape, alltrue, sometrue
 from Numeric import Int8, UInt8, Int16, UInt16, Int32, UInt32, \
      Float32, Float64, Complex32, Complex64, Float, Int, Complex
-from matplotlib._isnan import isnan64 as _isnan
+from numpy import isnan as _isnan
 
 class _TypeNamespace:
     """Numeric compatible type aliases for use with extension functions."""
