@@ -6,56 +6,60 @@ sessions
 
 Requirements:
 
-  tex
-
-  *Agg backends: dvipng
-
-  PS backend: latex w/ psfrag, dvips, and Ghostscript 8.51
+* latex
+* \*Agg backends: dvipng
+* PS backend: latex w/ psfrag, dvips, and Ghostscript 8.51
   (older versions do not work properly)
 
 Backends:
 
-  Only supported on *Agg and PS backends currently
-
+* \*Agg
+* PS
+* PDF
 
 For raster output, you can get RGBA numpy arrays from TeX expressions
-as follows
+as follows::
 
   texmanager = TexManager()
-  s = r'\TeX\ is Number $\displaystyle\sum_{n=1}^\infty\frac{-e^{i\pi}}{2^n}$!'
+  s = '\\TeX\\ is Number $\\displaystyle\\sum_{n=1}^\\infty\\frac{-e^{i\pi}}{2^n}$!'
   Z = self.texmanager.get_rgba(s, size=12, dpi=80, rgb=(1,0,0))
 
 To enable tex rendering of all text in your matplotlib figure, set
 text.usetex in your matplotlibrc file (http://matplotlib.sf.net/matplotlibrc)
-or include these two lines in your script:
-from matplotlib import rc
-rc('text', usetex=True)
+or include these two lines in your script::
+
+  from matplotlib import rc
+  rc('text', usetex=True)
 
 """
 
 import copy, glob, md5, os, shutil, sys, warnings
-import numpy as npy
+import distutils.version
+import numpy as np
 import matplotlib as mpl
 from matplotlib import rcParams
-from matplotlib._image import readpng
+from matplotlib._png import read_png
 
 DEBUG = False
 
 if sys.platform.startswith('win'): cmd_split = '&'
 else: cmd_split = ';'
 
-def get_dvipng_version():
+def dvipng_hack_alpha():
     stdin, stdout = os.popen4('dvipng -version')
     for line in stdout:
         if line.startswith('dvipng '):
             version = line.split()[-1]
             mpl.verbose.report('Found dvipng version %s'% version,
                 'helpful')
-            return version
+            version = distutils.version.LooseVersion(version)
+            return version < distutils.version.LooseVersion('1.6')
     raise RuntimeError('Could not obtain dvipng version')
 
 
+
 class TexManager:
+
     """
     Convert strings to dvi files using TeX, caching the results to a
     working dir
@@ -76,7 +80,7 @@ WARNING: found a TeX cache dir in the deprecated location "%s".
     if not os.path.exists(texcache):
         os.mkdir(texcache)
 
-    dvipngVersion = get_dvipng_version()
+    _dvipng_hack_alpha = dvipng_hack_alpha()
 
     # mappable cache of
     rgba_arrayd = {}
@@ -155,6 +159,9 @@ WARNING: found a TeX cache dir in the deprecated location "%s".
                                          r'\usepackage{textcomp}'])
 
     def get_basefile(self, tex, fontsize, dpi=None):
+        """
+        returns a filename based on a hash of the string, fontsize, and dpi
+        """
         s = ''.join([tex, self.get_font_config(), '%f'%fontsize,
                      self.get_custom_preamble(), str(dpi or '')])
         # make sure hash is consistent for all strings, regardless of encoding:
@@ -162,7 +169,7 @@ WARNING: found a TeX cache dir in the deprecated location "%s".
         return os.path.join(self.texcache, md5.md5(bytes).hexdigest())
 
     def get_font_config(self):
-        "Reinitializes self if rcParams self depends on have changed."
+        """Reinitializes self if relevant rcParams on have changed."""
         if self._rc_cache is None:
             self._rc_cache = dict([(k,None) for k in self._rc_cache_keys])
         changed = [par for par in self._rc_cache_keys if rcParams[par] != \
@@ -181,12 +188,16 @@ WARNING: found a TeX cache dir in the deprecated location "%s".
         return self._fontconfig
 
     def get_font_preamble(self):
+        """
+        returns a string containing font configuration for the tex preamble
+        """
         return self._font_preamble
 
     def get_custom_preamble(self):
+        """returns a string containing user additions to the tex preamble"""
         return '\n'.join(rcParams['text.latex.preamble'])
 
-    def get_shell_cmd(self, *args):
+    def _get_shell_cmd(self, *args):
         """
         On windows, changing directories can be complicated by the presence of
         multiple drives. get_shell_cmd deals with this issue.
@@ -199,6 +210,11 @@ WARNING: found a TeX cache dir in the deprecated location "%s".
         return ' && '.join(command)
 
     def make_tex(self, tex, fontsize):
+        """
+        Generate a tex file to render the tex string at a specific font size
+
+        returns the file name
+        """
         basefile = self.get_basefile(tex, fontsize)
         texfile = '%s.tex'%basefile
         fh = file(texfile, 'w')
@@ -241,14 +257,18 @@ WARNING: found a TeX cache dir in the deprecated location "%s".
         return texfile
 
     def make_dvi(self, tex, fontsize):
+        """
+        generates a dvi file containing latex's layout of tex string
 
+        returns the file name
+        """
         basefile = self.get_basefile(tex, fontsize)
         dvifile = '%s.dvi'% basefile
 
         if DEBUG or not os.path.exists(dvifile):
             texfile = self.make_tex(tex, fontsize)
             outfile = basefile+'.output'
-            command = self.get_shell_cmd('cd "%s"'% self.texcache,
+            command = self._get_shell_cmd('cd "%s"'% self.texcache,
                             'latex -interaction=nonstopmode %s > "%s"'\
                             %(os.path.split(texfile)[-1], outfile))
             mpl.verbose.report(command, 'debug')
@@ -267,6 +287,11 @@ string:\n%s\nHere is the full report generated by LaTeX: \n\n'% repr(tex)) + fh.
         return dvifile
 
     def make_png(self, tex, fontsize, dpi):
+        """
+        generates a png file containing latex's rendering of tex string
+
+        returns the filename
+        """
         basefile = self.get_basefile(tex, fontsize, dpi)
         pngfile = '%s.png'% basefile
 
@@ -274,7 +299,7 @@ string:\n%s\nHere is the full report generated by LaTeX: \n\n'% repr(tex)) + fh.
         if DEBUG or not os.path.exists(pngfile):
             dvifile = self.make_dvi(tex, fontsize)
             outfile = basefile+'.output'
-            command = self.get_shell_cmd('cd "%s"' % self.texcache,
+            command = self._get_shell_cmd('cd "%s"' % self.texcache,
                         'dvipng -bg Transparent -D %s -T tight -o \
                         "%s" "%s" > "%s"'%(dpi, os.path.split(pngfile)[-1],
                         os.path.split(dvifile)[-1], outfile))
@@ -292,14 +317,18 @@ process the flowing file:\n%s\nHere is the full report generated by dvipng: \
         return pngfile
 
     def make_ps(self, tex, fontsize):
+        """
+        generates a postscript file containing latex's rendering of tex string
 
+        returns the file name
+        """
         basefile = self.get_basefile(tex, fontsize)
         psfile = '%s.epsf'% basefile
 
         if DEBUG or not os.path.exists(psfile):
             dvifile = self.make_dvi(tex, fontsize)
             outfile = basefile+'.output'
-            command = self.get_shell_cmd('cd "%s"'% self.texcache,
+            command = self._get_shell_cmd('cd "%s"'% self.texcache,
                         'dvips -q -E -o "%s" "%s" > "%s"'\
                         %(os.path.split(psfile)[-1],
                           os.path.split(dvifile)[-1], outfile))
@@ -317,6 +346,10 @@ process the flowing file:\n%s\nHere is the full report generated by dvipng: \
         return psfile
 
     def get_ps_bbox(self, tex, fontsize):
+        """
+        returns a list containing the postscript bounding box for latex's
+        rendering of the tex string
+        """
         psfile = self.make_ps(tex, fontsize)
         ps = file(psfile)
         for line in ps:
@@ -325,16 +358,43 @@ process the flowing file:\n%s\nHere is the full report generated by dvipng: \
         raise RuntimeError('Could not parse %s'%psfile)
 
     def get_grey(self, tex, fontsize=None, dpi=None):
+        """returns the alpha channel"""
         key = tex, self.get_font_config(), fontsize, dpi
         alpha = self.grey_arrayd.get(key)
 
         if alpha is None:
             pngfile = self.make_png(tex, fontsize, dpi)
-            X = readpng(os.path.join(self.texcache, pngfile))
+            X = read_png(os.path.join(self.texcache, pngfile))
 
-            if (self.dvipngVersion < '1.6') or rcParams['text.dvipnghack']:
-                # hack the alpha channel as described in comment above
-                alpha = npy.sqrt(1-X[:,:,0])
+            if rcParams['text.dvipnghack'] is not None:
+                hack = rcParams['text.dvipnghack']
+            else:
+                hack = self._dvipng_hack_alpha
+
+            if hack:
+                # hack the alpha channel
+                # dvipng assumed a constant background, whereas we want to
+                # overlay these rasters with antialiasing over arbitrary
+                # backgrounds that may have other figure elements under them.
+                # When you set dvipng -bg Transparent, it actually makes the
+                # alpha channel 1 and does the background compositing and
+                # antialiasing itself and puts the blended data in the rgb
+                # channels.  So what we do is extract the alpha information
+                # from the red channel, which is a blend of the default dvipng
+                # background (white) and foreground (black).  So the amount of
+                # red (or green or blue for that matter since white and black
+                # blend to a grayscale) is the alpha intensity.  Once we
+                # extract the correct alpha information, we assign it to the
+                # alpha channel properly and let the users pick their rgb.  In
+                # this way, we can overlay tex strings on arbitrary
+                # backgrounds with antialiasing
+                #
+                # red = alpha*red_foreground + (1-alpha)*red_background
+                #
+                # Since the foreground is black (0) and the background is
+                # white (1) this reduces to red = 1-alpha or alpha = 1-red
+                #alpha = npy.sqrt(1-X[:,:,0]) # should this be sqrt here?
+                alpha = 1-X[:,:,0]
             else:
                 alpha = X[:,:,-1]
 
@@ -344,28 +404,8 @@ process the flowing file:\n%s\nHere is the full report generated by dvipng: \
 
     def get_rgba(self, tex, fontsize=None, dpi=None, rgb=(0,0,0)):
         """
-        Return tex string as an rgba array
+        Returns latex's rendering of the tex string as an rgba array
         """
-        # dvipng assumes a constant background, whereas we want to
-        # overlay these rasters with antialiasing over arbitrary
-        # backgrounds that may have other figure elements under them.
-        # When you set dvipng -bg Transparent, it actually makes the
-        # alpha channel 1 and does the background compositing and
-        # antialiasing itself and puts the blended data in the rgb
-        # channels.  So what we do is extract the alpha information
-        # from the red channel, which is a blend of the default dvipng
-        # background (white) and foreground (black).  So the amount of
-        # red (or green or blue for that matter since white and black
-        # blend to a grayscale) is the alpha intensity.  Once we
-        # extract the correct alpha information, we assign it to the
-        # alpha channel properly and let the users pick their rgb.  In
-        # this way, we can overlay tex strings on arbitrary
-        # backgrounds with antialiasing
-        #
-        # red = alpha*red_foreground + (1-alpha)*red_background
-        #
-        # Since the foreground is black (0) and the background is
-        # white (1) this reduces to red = 1-alpha or alpha = 1-red
         if not fontsize: fontsize = rcParams['font.size']
         if not dpi: dpi = rcParams['savefig.dpi']
         r,g,b = rgb
@@ -375,7 +415,8 @@ process the flowing file:\n%s\nHere is the full report generated by dvipng: \
         if Z is None:
             alpha = self.get_grey(tex, fontsize, dpi)
 
-            Z = npy.zeros((X.shape[0], X.shape[1], 4), npy.float)
+            Z = np.zeros((alpha.shape[0], alpha.shape[1], 4), np.float)
+
             Z[:,:,0] = r
             Z[:,:,1] = g
             Z[:,:,2] = b
