@@ -2,20 +2,18 @@
 Figure and Axes text
 """
 from __future__ import division
-import re, math
+import math
 
-import numpy as npy
+import numpy as np
 
-import matplotlib
-from matplotlib import verbose
 from matplotlib import cbook
 from matplotlib import rcParams
 import artist
 from artist import Artist
-from cbook import enumerate, is_string_like, maxdict, is_numlike
+from cbook import is_string_like, maxdict
 from font_manager import FontProperties
 from patches import bbox_artist, YAArrow
-from transforms import lbwh_to_bbox, bbox_all, identity_transform
+from transforms import Affine2D, Bbox
 from lines import Line2D
 
 import matplotlib.nxutils as nxutils
@@ -40,41 +38,42 @@ def get_rotation(rotation):
         angle = float(rotation)
     return angle%360
 
-_unit_box = lbwh_to_bbox(0,0,1,1)
-
 # these are not available for the object inspector until after the
 # class is build so we define an initial set here for the init
 # function and they will be overridden after object defn
 artist.kwdocd['Text'] =  """\
-    alpha: float
-    animated: [True | False]
-    backgroundcolor: any matplotlib color
-    bbox: rectangle prop dict plus key 'pad' which is a pad in points
-    clip_box: a matplotlib.transform.Bbox instance
-    clip_on: [True | False]
-    color: any matplotlib color
-    family: [ 'serif' | 'sans-serif' | 'cursive' | 'fantasy' | 'monospace' ]
-    figure: a matplotlib.figure.Figure instance
-    fontproperties: a matplotlib.font_manager.FontProperties instance
-    horizontalalignment or ha: [ 'center' | 'right' | 'left' ]
-    label: any string
-    linespacing: float
-    lod: [True | False]
-    multialignment: ['left' | 'right' | 'center' ]
-    name or fontname: string eg, ['Sans' | 'Courier' | 'Helvetica' ...]
-    position: (x,y)
-    rotation: [ angle in degrees 'vertical' | 'horizontal'
-    size or fontsize: [ size in points | relative size eg 'smaller', 'x-large' ]
-    style or fontstyle: [ 'normal' | 'italic' | 'oblique']
-    text: string
-    transform: a matplotlib.transform transformation instance
-    variant: [ 'normal' | 'small-caps' ]
-    verticalalignment or va: [ 'center' | 'top' | 'bottom' | 'baseline' ]
-    visible: [True | False]
-    weight or fontweight: [ 'normal' | 'bold' | 'heavy' | 'light' | 'ultrabold' | 'ultralight']
-    x: float
-    y: float
-    zorder: any number
+    ========================== =========================================================================
+    alpha                      float
+    ========================== =========================================================================
+    animated                   [True | False]
+    backgroundcolor            any matplotlib color
+    bbox                       rectangle prop dict plus key 'pad' which is a pad in points
+    clip_box                   a matplotlib.transform.Bbox instance
+    clip_on                    [True | False]
+    color                      any matplotlib color
+    family                     [ 'serif' | 'sans-serif' | 'cursive' | 'fantasy' | 'monospace' ]
+    figure                     a matplotlib.figure.Figure instance
+    fontproperties             a matplotlib.font_manager.FontProperties instance
+    horizontalalignment or ha  [ 'center' | 'right' | 'left' ]
+    label                      any string
+    linespacing                float
+    lod                        [True | False]
+    multialignment             ['left' | 'right' | 'center' ]
+    name or fontname           string eg, ['Sans' | 'Courier' | 'Helvetica' ...]
+    position                   (x,y)
+    rotation                   [ angle in degrees 'vertical' | 'horizontal'
+    size or fontsize           [ size in points | relative size eg 'smaller', 'x-large' ]
+    style or fontstyle         [ 'normal' | 'italic' | 'oblique']
+    text                       string
+    transform                  a matplotlib.transform transformation instance
+    variant                    [ 'normal' | 'small-caps' ]
+    verticalalignment or va    [ 'center' | 'top' | 'bottom' | 'baseline' ]
+    visible                    [True | False]
+    weight or fontweight       [ 'normal' | 'bold' | 'heavy' | 'light' | 'ultrabold' | 'ultralight']
+    x                          float
+    y                          float
+    zorder                     any number
+    ========================== =========================================================================
     """
 
 class Text(Artist):
@@ -98,7 +97,8 @@ class Text(Artist):
                  **kwargs
                  ):
         """
-        Create a Text instance at x,y with string text.
+        Create a :class:`~matplotlib.text.Text` instance at *x*, *y*
+        with string *text*.
 
         Valid kwargs are
         %(Text)s
@@ -151,7 +151,7 @@ class Text(Artist):
     def _get_xy_display(self):
         'get the (possibly unit converted) transformed x,y in display coords'
         x, y = self.get_position()
-        return self.get_transform().xy_tup((x,y))
+        return self.get_transform().transform_point((x,y))
 
     def _get_multialignment(self):
         if self._multialignment is not None: return self._multialignment
@@ -174,21 +174,19 @@ class Text(Artist):
         self._linespacing = other._linespacing
 
     def _get_layout(self, renderer):
-
-        # layout the xylocs in display coords as if angle = zero and
-        # then rotate them around self._x, self._y
-        #return _unit_box
         key = self.get_prop_tup()
         if self.cached.has_key(key): return self.cached[key]
-        horizLayout = []
-        thisx, thisy = self._get_xy_display()
-        width = 0
-        height = 0
 
-        xmin, ymin = thisx, thisy
+        horizLayout = []
+
+        thisx, thisy  = 0.0, 0.0
+        xmin, ymin    = 0.0, 0.0
+        width, height = 0.0, 0.0
         lines = self._text.split('\n')
 
-        whs = []
+        whs = np.zeros((len(lines), 2))
+        horizLayout = np.zeros((len(lines), 4))
+
         # Find full vertical extent of font,
         # including ascenders and descenders:
         tmp, heightt, bl = renderer.get_text_width_height_descent(
@@ -196,49 +194,47 @@ class Text(Artist):
         offsety = heightt * self._linespacing
 
         baseline = None
-        for line in lines:
+        for i, line in enumerate(lines):
             w, h, d = renderer.get_text_width_height_descent(
                 line, self._fontproperties, ismath=self.is_math_text(line))
             if baseline is None:
                 baseline = h - d
-            whs.append( (w,h) )
-            horizLayout.append((line, thisx, thisy, w, h))
+            whs[i] = w, h
+            horizLayout[i] = thisx, thisy, w, h
             thisy -= offsety
             width = max(width, w)
 
-        ymin = horizLayout[-1][2]
-        ymax = horizLayout[0][2] + horizLayout[0][-1]
+        ymin = horizLayout[-1][1]
+        ymax = horizLayout[0][1] + horizLayout[0][3]
         height = ymax-ymin
-
         xmax = xmin + width
-        # get the rotation matrix
-        M = self.get_rotation_matrix(xmin, ymin)
 
-        # the corners of the unrotated bounding box
-        cornersHoriz = ( (xmin, ymin), (xmin, ymax), (xmax, ymax), (xmax, ymin) )
-        offsetLayout = []
+        # get the rotation matrix
+        M = Affine2D().rotate_deg(self.get_rotation())
+
+        offsetLayout = np.zeros((len(lines), 2))
+        offsetLayout[:] = horizLayout[:, 0:2]
         # now offset the individual text lines within the box
         if len(lines)>1: # do the multiline aligment
             malign = self._get_multialignment()
-            for line, thisx, thisy, w, h in horizLayout:
-                if malign=='center': offsetx = width/2.0-w/2.0
-                elif malign=='right': offsetx = width-w
-                else: offsetx = 0
-                thisx += offsetx
-                offsetLayout.append( (thisx, thisy ))
-        else: # no additional layout needed
-            offsetLayout = [ (thisx, thisy) for line, thisx, thisy, w, h in horizLayout]
+            if malign == 'center':
+                offsetLayout[:, 0] += width/2.0 - horizLayout[:, 2] / 2.0
+            elif malign == 'right':
+                offsetLayout[:, 0] += width - horizLayout[:, 2]
 
+        # the corners of the unrotated bounding box
+        cornersHoriz = np.array(
+            [(xmin, ymin), (xmin, ymax), (xmax, ymax), (xmax, ymin)],
+            np.float_)
         # now rotate the bbox
+        cornersRotated = M.transform(cornersHoriz)
 
-        cornersRotated = [npy.dot(M,npy.array([[thisx],[thisy],[1]])) for thisx, thisy in cornersHoriz]
-
-        txs = [float(v[0][0]) for v in cornersRotated]
-        tys = [float(v[1][0]) for v in cornersRotated]
+        txs = cornersRotated[:, 0]
+        tys = cornersRotated[:, 1]
 
         # compute the bounds of the rotated box
-        xmin, xmax = min(txs), max(txs)
-        ymin, ymax = min(tys), max(tys)
+        xmin, xmax = txs.min(), txs.max()
+        ymin, ymax = tys.min(), tys.max()
         width  = xmax - xmin
         height = ymax - ymin
 
@@ -248,42 +244,30 @@ class Text(Artist):
 
         # compute the text location in display coords and the offsets
         # necessary to align the bbox with that location
-        tx, ty = self._get_xy_display()
+        if halign=='center':  offsetx = (xmin + width/2.0)
+        elif halign=='right': offsetx = (xmin + width)
+        else: offsetx = xmin
 
-        if halign=='center':  offsetx = tx - (xmin + width/2.0)
-        elif halign=='right': offsetx = tx - (xmin + width)
-        else: offsetx = tx - xmin
+        if valign=='center': offsety = (ymin + height/2.0)
+        elif valign=='top': offsety  = (ymin + height)
+        elif valign=='baseline': offsety = (ymin + height) - baseline
+        else: offsety = ymin
 
-        if valign=='center': offsety = ty - (ymin + height/2.0)
-        elif valign=='top': offsety  = ty - (ymin + height)
-        elif valign=='baseline': offsety = ty - (ymin + height) + baseline
-        else: offsety = ty - ymin
-
-        xmin += offsetx
-        xmax += offsetx
-        ymin += offsety
-        ymax += offsety
-
-        bbox = lbwh_to_bbox(xmin, ymin, width, height)
+        xmin -= offsetx
+        ymin -= offsety
 
+        bbox = Bbox.from_bounds(xmin, ymin, width, height)
 
         # now rotate the positions around the first x,y position
-        xys = [npy.dot(M,npy.array([[thisx],[thisy],[1]])) for thisx, thisy in offsetLayout]
+        xys = M.transform(offsetLayout)
+        xys -= (offsetx, offsety)
 
-
-        tx = [float(v[0][0])+offsetx for v in xys]
-        ty = [float(v[1][0])+offsety for v in xys]
-
-        # now inverse transform back to data coords
-        xys = [self.get_transform().inverse_xy_tup( xy ) for xy in zip(tx, ty)]
-
-        xs, ys = zip(*xys)
+        xs, ys = xys[:, 0], xys[:, 1]
 
         ret = bbox, zip(lines, whs, xs, ys)
         self.cached[key] = ret
         return ret
 
-
     def set_bbox(self, rectprops):
         """
         Draw a bounding box around self.  rect props are any settable
@@ -306,9 +290,7 @@ class Text(Artist):
         gc.set_foreground(self._color)
         gc.set_alpha(self._alpha)
         if self.get_clip_on():
-            gc.set_clip_rectangle(self.clipbox.get_bounds())
-
-
+            gc.set_clip_rectangle(self.clipbox)
 
         if self._bbox:
             bbox_artist(self, renderer, self._bbox)
@@ -316,10 +298,14 @@ class Text(Artist):
 
         bbox, info = self._get_layout(renderer)
         trans = self.get_transform()
+        posx, posy = self.get_position()
+        posx, posy = trans.transform_point((posx, posy))
+        canvasw, canvash = renderer.get_canvas_width_height()
+
         if rcParams['text.usetex']:
-            canvasw, canvash = renderer.get_canvas_width_height()
             for line, wh, x, y in info:
-                x, y = trans.xy_tup((x, y))
+                x = x + posx
+                y = y + posy
                 if renderer.flipy():
                     y = canvash-y
 
@@ -328,10 +314,9 @@ class Text(Artist):
             return
 
         for line, wh, x, y in info:
-            x, y = trans.xy_tup((x, y))
-
+            x = x + posx
+            y = y + posy
             if renderer.flipy():
-                canvasw, canvash = renderer.get_canvas_width_height()
                 y = canvash-y
 
             renderer.draw_text(gc, x, y, line,
@@ -390,7 +375,7 @@ class Text(Artist):
 
     def _get_xy_display(self):
         'get the (possibly unit converted) transformed x,y in display coords'
-        return self.get_transform().xy_tup((self._x, self._y))
+        return self.get_transform().transform_point((self._x, self._y))
 
     def get_position(self):
         "Return x, y as tuple"
@@ -410,7 +395,7 @@ class Text(Artist):
         return (x, y, self._text, self._color,
                 self._verticalalignment, self._horizontalalignment,
                 hash(self._fontproperties), self._rotation,
-                self.get_transform().as_vec6_val(),
+                self._renderer.dpi, id(self._renderer)
                 )
 
     def get_text(self):
@@ -427,10 +412,10 @@ class Text(Artist):
 
     def get_window_extent(self, renderer=None):
         #return _unit_box
-        if not self.get_visible(): return _unit_box
+        if not self.get_visible(): return Bbox.unit()
         if self._text == '':
             tx, ty = self._get_xy_display()
-            return lbwh_to_bbox(tx,ty,0,0)
+            return Bbox.from_bounds(tx,ty,0,0)
 
         if renderer is not None:
             self._renderer = renderer
@@ -439,31 +424,11 @@ class Text(Artist):
 
         angle = self.get_rotation()
         bbox, info = self._get_layout(self._renderer)
+        x, y = self.get_position()
+        x, y = self.get_transform().transform_point((x, y))
+        bbox = bbox.translated(x, y)
         return bbox
 
-
-
-    def get_rotation_matrix(self, x0, y0):
-
-        theta = npy.pi/180.0*self.get_rotation()
-        # translate x0,y0 to origin
-        Torigin = npy.array([ [1, 0, -x0],
-                           [0, 1, -y0],
-                           [0, 0, 1  ]])
-
-        # rotate by theta
-        R = npy.array([ [npy.cos(theta),  -npy.sin(theta), 0],
-                     [npy.sin(theta), npy.cos(theta), 0],
-                     [0,           0,          1]])
-
-        # translate origin back to x0,y0
-        Tback = npy.array([ [1, 0, x0],
-                         [0, 1, y0],
-                         [0, 0, 1  ]])
-
-
-        return npy.dot(npy.dot(Tback,R), Torigin)
-
     def set_backgroundcolor(self, color):
         """
         Set the background color of the text by updating the bbox (see set_bbox for more info)
@@ -684,50 +649,51 @@ artist.kwdocd['Text'] = artist.kwdoc(Text)
 
 class TextWithDash(Text):
     """
-    This is basically a Text with a dash (drawn with a Line2D)
-    before/after it. It is intended to be a drop-in replacement
-    for Text, and should behave identically to Text when
-    dashlength=0.0.
+    This is basically a :class:`~matplotlib.text.Text` with a dash
+    (drawn with a :class:`~matplotlib.lines.Line2D`) before/after
+    it. It is intended to be a drop-in replacement for
+    :class:`~matplotlib.text.Text`, and should behave identically to
+    it when *dashlength* = 0.0.
 
     The dash always comes between the point specified by
-    set_position() and the text. When a dash exists, the
-    text alignment arguments (horizontalalignment,
-    verticalalignment) are ignored.
+    :meth:`~matplotlib.text.Text.set_position` and the text. When a
+    dash exists, the text alignment arguments (*horizontalalignment*,
+    *verticalalignment*) are ignored.
 
-    dashlength is the length of the dash in canvas units.
-    (default=0.0).
+    *dashlength* is the length of the dash in canvas units.
+    (default = 0.0).
 
-    dashdirection is one of 0 or 1, npy.where 0 draws the dash
-    after the text and 1 before.
-    (default=0).
+    *dashdirection* is one of 0 or 1, where 0 draws the dash after the
+    text and 1 before.  (default = 0).
 
-    dashrotation specifies the rotation of the dash, and
-    should generally stay None. In this case
-    self.get_dashrotation() returns self.get_rotation().
-    (I.e., the dash takes its rotation from the text's
-    rotation). Because the text center is projected onto
-    the dash, major deviations in the rotation cause
+    *dashrotation* specifies the rotation of the dash, and should
+    generally stay *None*. In this case
+    :meth:`~matplotlib.text.TextWithDash.get_dashrotation` returns
+    :meth:`~matplotlib.text.Text.get_rotation`.  (I.e., the dash takes
+    its rotation from the text's rotation). Because the text center is
+    projected onto the dash, major deviations in the rotation cause
     what may be considered visually unappealing results.
-    (default=None).
+    (default = *None*)
 
-    dashpad is a padding length to add (or subtract) space
+    *dashpad* is a padding length to add (or subtract) space
     between the text and the dash, in canvas units.
-    (default=3).
+    (default = 3)
 
-    dashpush "pushes" the dash and text away from the point
-    specified by set_position() by the amount in canvas units.
-    (default=0)
+    *dashpush* "pushes" the dash and text away from the point
+    specified by :meth:`~matplotlib.text.Text.set_position` by the
+    amount in canvas units.  (default = 0)
 
-    NOTE: The alignment of the two objects is based on the
-    bbox of the Text, as obtained by get_window_extent().
-    This, in turn, appears to depend on the font metrics
-    as given by the rendering backend. Hence the quality
-    of the "centering" of the label text with respect to
-    the dash varies depending on the backend used.
+    *NOTE*: The alignment of the two objects is based on the bounding
+    box of the :class:`~matplotlib.text.Text`, as obtained by
+    :meth:`~matplotlib.artist.Artist.get_window_extent`.  This, in
+    turn, appears to depend on the font metrics as given by the
+    rendering backend. Hence the quality of the "centering" of the
+    label text with respect to the dash varies depending on the
+    backend used.
 
-    NOTE2: I'm not sure that I got the get_window_extent()
-    right, or whether that's sufficient for providing the
-    object bbox.
+    *NOTE 2*: I'm not sure that I got the
+    :meth:`~matplotlib.text.TextWithDash.get_window_extent` right, or
+    whether that's sufficient for providing the object bounding box.
     """
     __name__ = 'textwithdash'
 
@@ -785,18 +751,17 @@ class TextWithDash(Text):
 
     def get_prop_tup(self):
         """
-        Return a hashable tuple of properties
+        Return a hashable tuple of properties.
 
         Not intended to be human readable, but useful for backends who
         want to cache derived information about text (eg layouts) and
-        need to know if the text has changed
+        need to know if the text has changed.
         """
         props = [p for p in Text.get_prop_tup(self)]
         props.extend([self._x, self._y, self._dashlength, self._dashdirection, self._dashrotation, self._dashpad, self._dashpush])
         return tuple(props)
 
     def draw(self, renderer):
-        self.cached = dict()
         self.update_coords(renderer)
         Text.draw(self, renderer)
         if self.get_dashlength() > 0.0:
@@ -822,20 +787,21 @@ class TextWithDash(Text):
         dashpush = self.get_dashpush()
 
         angle = get_rotation(dashrotation)
-        theta = npy.pi*(angle/180.0+dashdirection-1)
-        cos_theta, sin_theta = npy.cos(theta), npy.sin(theta)
+        theta = np.pi*(angle/180.0+dashdirection-1)
+        cos_theta, sin_theta = np.cos(theta), np.sin(theta)
 
         transform = self.get_transform()
 
         # Compute the dash end points
         # The 'c' prefix is for canvas coordinates
-        cxy = npy.array(transform.xy_tup((dashx, dashy)))
-        cd = npy.array([cos_theta, sin_theta])
+        cxy = transform.transform_point((dashx, dashy))
+        cd = np.array([cos_theta, sin_theta])
         c1 = cxy+dashpush*cd
         c2 = cxy+(dashpush+dashlength)*cd
 
-        (x1, y1) = transform.inverse_xy_tup(tuple(c1))
-        (x2, y2) = transform.inverse_xy_tup(tuple(c2))
+        inverse = transform.inverted()
+        (x1, y1) = inverse.transform_point(tuple(c1))
+        (x2, y2) = inverse.transform_point(tuple(c2))
         self.dashline.set_data((x1, x2), (y1, y2))
 
         # We now need to extend this vector out to
@@ -853,7 +819,7 @@ class TextWithDash(Text):
         # but I don't grok the transformation stuff
         # well enough yet.
         we = Text.get_window_extent(self, renderer=renderer)
-        w, h = we.width(), we.height()
+        w, h = we.width, we.height
         # Watch for zeros
         if sin_theta == 0.0:
             dx = w
@@ -868,21 +834,18 @@ class TextWithDash(Text):
             if dy > h or dy < -h:
                 dy = h
                 dx = h/tan_theta
-        cwd = npy.array([dx, dy])/2
-        cwd *= 1+dashpad/npy.sqrt(npy.dot(cwd,cwd))
+        cwd = np.array([dx, dy])/2
+        cwd *= 1+dashpad/np.sqrt(np.dot(cwd,cwd))
         cw = c2+(dashdirection*2-1)*cwd
 
-
-
-        newx, newy = transform.inverse_xy_tup(tuple(cw))
-
+        newx, newy = inverse.transform_point(tuple(cw))
         self._x, self._y = newx, newy
 
         # Now set the window extent
         # I'm not at all sure this is the right way to do this.
         we = Text.get_window_extent(self, renderer=renderer)
-        self._twd_window_extent = we.deepcopy()
-        self._twd_window_extent.update(((c1[0], c1[1]),), False)
+        self._twd_window_extent = we.frozen()
+        self._twd_window_extent.update_from_data_xy(np.array([c1]), False)
 
         # Finally, make text align center
         Text.set_horizontalalignment(self, 'center')
@@ -1012,8 +975,10 @@ artist.kwdocd['TextWithDash'] = artist.kwdoc(TextWithDash)
 
 class Annotation(Text):
     """
-    A Text class to make annotating things in the figure: Figure,
-    Axes, Point, Rectangle, etc... easier
+    A :class:`~matplotlib.text.Text` class to make annotating things
+    in the figure, such as :class:`~matplotlib.figure.Figure`,
+    :class:`~matplotlib.axes.Axes`,
+    :class:`~matplotlib.patches.Rectangle`, etc., easier.
     """
     def __str__(self):
         return "Annotation(%g,%g,%s)"%(self.xy[0],self.xy[1],self._text)
@@ -1024,44 +989,56 @@ class Annotation(Text):
                  arrowprops=None,
                  **kwargs):
         """
-        Annotate the x,y point xy with text s at x,y location xytext
-        (xytext if None defaults to xy and textcoords if None defaults
-        to xycoords).
+        Annotate the *x*, *y* point *xy* with text *s* at *x*, *y*
+        location *xytext*.  (If *xytext* = *None*, defaults to *xy*,
+        and if *textcoords* = *None*, defaults to *xycoords*).
 
-        arrowprops, if not None, is a dictionary of line properties
-        (see matplotlib.lines.Line2D) for the arrow that connects
+        *arrowprops*, if not *None*, is a dictionary of line properties
+        (see :class:`matplotlib.lines.Line2D`) for the arrow that connects
         annotation to the point.   Valid keys are
 
-          - width : the width of the arrow in points
-          - frac  : the fraction of the arrow length occupied by the head
-          - headwidth : the width of the base of the arrow head in points
-          - shrink: often times it is convenient to have the arrowtip
-            and base a bit away from the text and point being
-            annotated.  If d is the distance between the text and
-            annotated point, shrink will shorten the arrow so the tip
-            and base are shink percent of the distance d away from the
-            endpoints.  ie, shrink=0.05 is 5%%
-          - any key for matplotlib.patches.polygon
+        =========   ===========================================================
+           Key                            Description
+        =========   ===========================================================
+        width       the width of the arrow in points
+        frac        the fraction of the arrow length occupied by the head
+        headwidth   the width of the base of the arrow head in points
+        shrink      oftentimes it is convenient to have the arrowtip
+                    and base a bit away from the text and point being
+                    annotated.  If *d* is the distance between the text and
+                    annotated point, shrink will shorten the arrow so the tip
+                    and base are shink percent of the distance *d* away from the
+                    endpoints.  ie, ``shrink=0.05 is 5%%``
+        ?           any key for :class:`matplotlib.patches.polygon`
+        =========   ===========================================================
 
-        xycoords and textcoords are strings that indicate the
-        coordinates of xy and xytext.
+        *xycoords* and *textcoords* are strings that indicate the
+        coordinates of *xy* and *xytext*.
 
-           'figure points'   : points from the lower left corner of the figure
-           'figure pixels'   : pixels from the lower left corner of the figure                           'figure fraction' : 0,0 is lower left of figure and 1,1 is upper, right
-           'axes points'     : points from lower left corner of axes
-           'axes pixels'     : pixels from lower left corner of axes
-           'axes fraction'   : 0,1 is lower left of axes and 1,1 is upper right
-           'data'            : use the coordinate system of the object being annotated (default)
-           'offset points'     : Specify an offset (in points) from the xy value
-           'polar'           : you can specify theta, r for the annotation, even
-                               in cartesian plots.  Note that if you
-                               are using a polar axes, you do not need
-                               to specify polar for the coordinate
-                               system since that is the native"data" coordinate system.
+        =================   ===================================================
+             Property                           Description
+        =================   ===================================================
+        'figure points'     points from the lower left corner of the figure
+        'figure pixels'     pixels from the lower left corner of the figure
+        'figure fraction'   0,0 is lower left of figure and 1,1 is upper, right
+        'axes points'       points from lower left corner of axes
+        'axes pixels'       pixels from lower left corner of axes
+        'axes fraction'     0,1 is lower left of axes and 1,1 is upper right
+        'data'              use the coordinate system of the object being
+                            annotated (default)
+        'offset points'     Specify an offset (in points) from the *xy* value
 
-        If a points or pixels option is specified, values will be
-        added to the left, bottom and if negative, values will be
-        subtracted from the top, right.  Eg,
+        'polar'             you can specify *theta*, *r* for the annotation,
+                            even in cartesian plots.  Note that if you
+                            are using a polar axes, you do not need
+                            to specify polar for the coordinate
+                            system since that is the native "data" coordinate
+                            system.
+        =================   ===================================================
+
+        If a 'points' or 'pixels' option is specified, values will be
+        added to the bottom-left and if negative, values will be
+        subtracted from the top-right.  Eg::
 
           # 10 points to the right of the left border of the axes and
           # 5 points below the top border
@@ -1094,6 +1071,12 @@ class Annotation(Text):
         return t,tinfo
 
 
+    def set_figure(self, fig):
+
+        if self.arrow is not None:
+            self.arrow.set_figure(fig)
+        Artist.set_figure(self, fig)
+
     def set_clip_box(self, clipbox):
         """
         Set the artist's clip Bbox
@@ -1107,19 +1090,19 @@ class Annotation(Text):
             trans = self.axes.transData
             x = float(self.convert_xunits(x))
             y = float(self.convert_yunits(y))
-            return trans.xy_tup((x,y))
+            return trans.transform_point((x, y))
         elif s=='offset points':
             # convert the data point
             dx, dy = self.xy
 
             # prevent recursion
             if self.xycoords == 'offset points':
-               return self._get_xy(dx, dy, 'data')
+                return self._get_xy(dx, dy, 'data')
 
             dx, dy = self._get_xy(dx, dy, self.xycoords)
 
             # convert the offset
-            dpi = self.figure.dpi.get()
+            dpi = self.figure.get_dpi()
             x *= dpi/72.
             y *= dpi/72.
 
@@ -1130,14 +1113,14 @@ class Annotation(Text):
             return x, y
         elif s=='polar':
             theta, r = x, y
-            x = r*npy.cos(theta)
-            y = r*npy.sin(theta)
+            x = r*np.cos(theta)
+            y = r*np.sin(theta)
             trans = self.axes.transData
-            return trans.xy_tup((x,y))
+            return trans.transform_point((x,y))
         elif s=='figure points':
             #points from the lower left corner of the figure
-            dpi = self.figure.dpi.get()
-            l,b,w,h = self.figure.bbox.get_bounds()
+            dpi = self.figure.dpi
+            l,b,w,h = self.figure.bbox.bounds
             r = l+w
             t = b+h
 
@@ -1150,7 +1133,7 @@ class Annotation(Text):
             return x,y
         elif s=='figure pixels':
             #pixels from the lower left corner of the figure
-            l,b,w,h = self.figure.bbox.get_bounds()
+            l,b,w,h = self.figure.bbox.bounds
             r = l+w
             t = b+h
             if x<0:
@@ -1161,11 +1144,11 @@ class Annotation(Text):
         elif s=='figure fraction':
             #(0,0) is lower left, (1,1) is upper right of figure
             trans = self.figure.transFigure
-            return trans.xy_tup((x,y))
+            return trans.transform_point((x,y))
         elif s=='axes points':
             #points from the lower left corner of the axes
-            dpi = self.figure.dpi.get()
-            l,b,w,h = self.axes.bbox.get_bounds()
+            dpi = self.figure.dpi
+            l,b,w,h = self.axes.bbox.bounds
             r = l+w
             t = b+h
             if x<0:
@@ -1180,7 +1163,7 @@ class Annotation(Text):
         elif s=='axes pixels':
             #pixels from the lower left corner of the axes
 
-            l,b,w,h = self.axes.bbox.get_bounds()
+            l,b,w,h = self.axes.bbox.bounds
             r = l+w
             t = b+h
             if x<0:
@@ -1195,7 +1178,7 @@ class Annotation(Text):
         elif s=='axes fraction':
             #(0,0) is lower left, (1,1) is upper right of axes
             trans = self.axes.transAxes
-            return trans.xy_tup((x,y))
+            return trans.transform_point((x, y))
 
 
     def update_positions(self, renderer):
@@ -1209,8 +1192,7 @@ class Annotation(Text):
 
         if self.arrowprops:
             x0, y0 = x, y
-            l,b,w,h = self.get_window_extent(renderer).get_bounds()
-            dpi = self.figure.dpi.get()
+            l,b,w,h = self.get_window_extent(renderer).bounds
             r = l+w
             t = b+h
             xc = 0.5*(l+r)
@@ -1237,7 +1219,7 @@ class Annotation(Text):
             dx = shrink*r*math.cos(theta)
             dy = shrink*r*math.sin(theta)
 
-            self.arrow = YAArrow(self.figure.dpi, (x0+dx,y0+dy), (x-dx, y-dy),
+            self.arrow = YAArrow(self.figure, (x0+dx,y0+dy), (x-dx, y-dy),
                             width=width, headwidth=headwidth, frac=frac,
                             **d)
             self.arrow.set_clip_box(self.get_clip_box())
@@ -1246,6 +1228,8 @@ class Annotation(Text):
         self.update_positions(renderer)
 
         if self.arrow is not None:
+            if self.arrow.figure is None and self.figure is not None:
+                self.arrow.figure = self.figure
             self.arrow.draw(renderer)
 
         Text.draw(self, renderer)
