@@ -1,35 +1,35 @@
+// To remove a gcc warning
+#ifdef _POSIX_C_SOURCE
+#undef _POSIX_C_SOURCE
+#endif
+
+#include "Python.h"
+#include <string>
+
 #include <iostream>
 #include <fstream>
 #include <cmath>
 #include <cstdio>
-#include <png.h>
-#include <string>
-#include "Python.h"
 
-#ifdef NUMARRAY
-#include "numarray/arrayobject.h"
-#else
-#ifdef NUMERIC
-#include "Numeric/arrayobject.h"
-#else
+#define PY_ARRAY_TYPES_PREFIX NumPy
 #include "numpy/arrayobject.h"
-#endif
-#endif
 
+#include "agg_color_rgba.h"
+#include "agg_conv_transform.h"
+#include "agg_image_accessors.h"
+#include "agg_path_storage.h"
 #include "agg_pixfmt_rgb.h"
 #include "agg_pixfmt_rgba.h"
-#include "agg_color_rgba.h"
-#include "agg_rendering_buffer.h"
 #include "agg_rasterizer_scanline_aa.h"
+#include "agg_renderer_scanline.h"
+#include "agg_rendering_buffer.h"
 #include "agg_scanline_bin.h"
-#include "agg_path_storage.h"
-#include "agg_conv_transform.h"
+#include "agg_scanline_bin.h"
+#include "agg_scanline_u.h"
+#include "agg_span_allocator.h"
 #include "agg_span_image_filter_rgb.h"
 #include "agg_span_image_filter_rgba.h"
 #include "agg_span_interpolator_linear.h"
-#include "agg_scanline_bin.h"
-#include "agg_scanline_u.h"
-#include "agg_renderer_scanline.h"
 #include "util/agg_color_conv_rgb8.h"
 #include "_image.h"
 #include "mplutils.h"
@@ -210,46 +210,51 @@ Image::as_rgba_str(const Py::Tuple& args, const Py::Dict& kwargs) {
 
   if (bufpair.second) delete [] bufpair.first;
   return ret;
-
-
 }
 
 
-char Image::buffer_argb32__doc__[] =
-"buffer = buffer_argb32()"
+char Image::color_conv__doc__[] =
+"numrows, numcols, buffer = color_conv(format)"
 "\n"
-"Return the image buffer as agbr32\n"
+"format 0(BGRA) or 1(ARGB)\n"
+"Convert image to format and return in a writable buffer\n"
 ;
 Py::Object
-Image::buffer_argb32(const Py::Tuple& args) {
-  //"Return the image object as argb32";
+Image::color_conv(const Py::Tuple& args) {
+  _VERBOSE("Image::color_conv");
 
-  _VERBOSE("RendererAgg::buffer_argb32");
-
-  args.verify_length(0);
+  args.verify_length(1);
+  int format = Py::Int(args[0]);
 
   int row_len = colsOut * 4;
+  PyObject* py_buffer = PyBuffer_New(row_len * rowsOut);
+  if (py_buffer ==NULL)
+    throw Py::MemoryError("Image::color_conv could not allocate memory");
 
-  unsigned char* buf_tmp = new unsigned char[row_len * rowsOut];
-  if (buf_tmp ==NULL)
-    throw Py::MemoryError("RendererAgg::buffer_argb32 could not allocate memory");
+  void* buf;
+  Py_ssize_t buffer_len;
+  int ret = PyObject_AsWriteBuffer(py_buffer, &buf, &buffer_len);
+  if (ret !=0)
+    throw Py::MemoryError("Image::color_conv could not allocate memory");
 
   agg::rendering_buffer rtmp;
-  rtmp.attach(buf_tmp, colsOut, rowsOut, row_len);
+  rtmp.attach(reinterpret_cast<unsigned char*>(buf), colsOut, rowsOut,
+	      row_len);
 
-  color_conv(&rtmp, rbufOut, agg::color_conv_rgba32_to_argb32());
-
-  //todo: how to do this with native CXX
-  //PyObject* o = Py_BuildValue("s#", buf_tmp, row_len * rowsOut);
-  PyObject* o = Py_BuildValue("lls#", rowsOut, colsOut,
-			      buf_tmp, row_len * rowsOut);
-  delete [] buf_tmp;
+  switch (format) {
+  case 0:
+    agg::color_conv(&rtmp, rbufOut, agg::color_conv_rgba32_to_bgra32());
+    break;
+  case 1:
+    agg::color_conv(&rtmp, rbufOut, agg::color_conv_rgba32_to_argb32());
+    break;
+  default:
+    throw Py::ValueError("Image::color_conv unknown format");
+  }
+  PyObject* o = Py_BuildValue("llN", rowsOut, colsOut, py_buffer);
   return Py::asObject(o);
-
-
 }
 
-
 char Image::buffer_rgba__doc__[] =
 "buffer = buffer_rgba()"
 "\n"
@@ -266,8 +271,6 @@ Image::buffer_rgba(const Py::Tuple& args) {
   PyObject* o = Py_BuildValue("lls#", rowsOut, colsOut,
 			      rbufOut, row_len * rowsOut);
   return Py::asObject(o);
-
-
 }
 
 char Image::reset_matrix__doc__[] =
@@ -289,6 +292,29 @@ Image::reset_matrix(const Py::Tuple& args) {
 
 }
 
+char Image::get_matrix__doc__[] =
+"(m11,m21,m12,m22,m13,m23) = get_matrix()\n"
+"\n"
+"Get the affine transformation matrix\n"
+"  /m11,m12,m13\\\n"
+"  /m21,m22,m23|\n"
+"  \\ 0 , 0 , 1 /"
+;
+
+Py::Object
+Image::get_matrix(const Py::Tuple& args) {
+  _VERBOSE("Image::get_matrix");
+
+  args.verify_length(0);
+
+  double m[6];
+  srcMatrix.store_to(m);
+  Py::Tuple ret(6);
+  for(int i=0;i<6;i++)
+    ret[i] = Py::Float(m[i]);
+  return ret;
+}
+
 char Image::resize__doc__[] =
 "resize(width, height, norm=1, radius=4.0)\n"
 "\n"
@@ -343,15 +369,13 @@ Image::resize(const Py::Tuple& args, const Py::Dict& kwargs) {
   imageMatrix.invert();
   interpolator_type interpolator(imageMatrix);
 
-  agg::span_allocator<agg::rgba8> sa;
+  typedef agg::span_allocator<agg::rgba8> span_alloc_type;
+  span_alloc_type sa;
   agg::rgba8 background(agg::rgba8(int(255*bg.r),
 				   int(255*bg.g),
 				   int(255*bg.b),
 				   int(255*bg.a)));
 
-
-
-
   // the image path
   agg::path_storage path;
   agg::int8u *bufferPad = NULL;
@@ -359,56 +383,10 @@ Image::resize(const Py::Tuple& args, const Py::Dict& kwargs) {
 
   double x0, y0, x1, y1;
 
-  if (interpolation==NEAREST) {
-    x0 = 0.0;
-    x1 = colsIn;
-    y0 = 0.0;
-    y1 = rowsIn;
-  }
-  else {
-    // if interpolation != nearest, create a new input buffer with the
-    // edges mirrored on all size.  Then new buffer size is colsIn+2 by
-    // rowsIn+2
-
-    x0 = 1.0;
-    x1 = colsIn+1;
-    y0 = 1.0;
-    y1 = rowsIn+1;
-
-
-    bufferPad = new agg::int8u[(rowsIn+2) * (colsIn+2) * BPP];
-    if (bufferPad ==NULL)
-      throw Py::MemoryError("Image::resize could not allocate memory");
-    rbufPad.attach(bufferPad, colsIn+2, rowsIn+2, (colsIn+2) * BPP);
-
-    pixfmt pixfpad(rbufPad);
-    renderer_base rbpad(pixfpad);
-
-    pixfmt pixfin(*rbufIn);
-    renderer_base rbin(pixfin);
-
-    rbpad.copy_from(*rbufIn, 0, 1, 1);
-
-    agg::rect_base<int> firstrow(0, 0, colsIn-1, 0);
-    rbpad.copy_from(*rbufIn, &firstrow, 1, 0);
-
-    agg::rect_base<int> lastrow(0, rowsIn-1, colsIn-1, rowsIn-1);
-    rbpad.copy_from(*rbufIn, &lastrow, 1, 2);
-
-    agg::rect_base<int> firstcol(0, 0, 0, rowsIn-1);
-    rbpad.copy_from(*rbufIn, &firstcol, 0, 1);
-
-    agg::rect_base<int> lastcol(colsIn-1, 0, colsIn-1, rowsIn-1);
-    rbpad.copy_from(*rbufIn, &lastcol, 2, 1);
-
-    rbpad.copy_pixel(0, 0, rbin.pixel(0,0) );
-    rbpad.copy_pixel(0, colsIn+1, rbin.pixel(0,colsIn-1) );
-    rbpad.copy_pixel(rowsIn+1, 0, rbin.pixel(rowsIn-1,0) );
-    rbpad.copy_pixel(rowsIn+1, colsIn+1, rbin.pixel(rowsIn-1,colsIn-1) );
-
-
-  }
-
+  x0 = 0.0;
+  x1 = colsIn;
+  y0 = 0.0;
+  y1 = rowsIn;
 
   path.move_to(x0, y0);
   path.line_to(x1, y0);
@@ -418,64 +396,99 @@ Image::resize(const Py::Tuple& args, const Py::Dict& kwargs) {
   agg::conv_transform<agg::path_storage> imageBox(path, srcMatrix);
   ras.add_path(imageBox);
 
+  typedef agg::wrap_mode_reflect reflect_type;
+  typedef agg::image_accessor_wrap<pixfmt, reflect_type, reflect_type> img_accessor_type;
+
+  pixfmt pixfmtin(*rbufIn);
+  img_accessor_type ia(pixfmtin);
   switch(interpolation)
     {
 
     case NEAREST:
       {
-	typedef agg::span_image_filter_rgba_nn<agg::rgba8,agg::order_rgba, interpolator_type> span_gen_type;
-	typedef agg::renderer_scanline_aa<renderer_base, span_gen_type> renderer_type;
-
-	span_gen_type sg(sa, *rbufIn, background, interpolator);
-	renderer_type ri(rb, sg);
+	typedef agg::span_image_filter_rgba_nn<img_accessor_type, interpolator_type> span_gen_type;
+	typedef agg::renderer_scanline_aa<renderer_base, span_alloc_type, span_gen_type> renderer_type;
+	span_gen_type sg(ia, interpolator);
+	renderer_type ri(rb, sa, sg);
 	agg::render_scanlines(ras, sl, ri);
       }
       break;
-        case BILINEAR:
-        case BICUBIC:
-        case SPLINE16:
-        case SPLINE36:
-        case HANNING:
-        case HAMMING:
-        case HERMITE:
-        case KAISER:
-        case QUADRIC:
-        case CATROM:
-        case GAUSSIAN:
-        case BESSEL:
-        case MITCHELL:
-        case SINC:
-        case LANCZOS:
-        case BLACKMAN:
-            {
-                agg::image_filter_lut filter;
-                switch(interpolation)
-                {
-                case BILINEAR:  filter.calculate(agg::image_filter_bilinear(), norm); break;
-                case BICUBIC:  filter.calculate(agg::image_filter_bicubic(), norm); break;
-                case SPLINE16:  filter.calculate(agg::image_filter_spline16(), norm); break;
-                case SPLINE36:  filter.calculate(agg::image_filter_spline36(), norm); break;
-                case HANNING:  filter.calculate(agg::image_filter_hanning(), norm); break;
-                case HAMMING:  filter.calculate(agg::image_filter_hamming(), norm); break;
-                case HERMITE:  filter.calculate(agg::image_filter_hermite(), norm); break;
-                case KAISER:  filter.calculate(agg::image_filter_kaiser(), norm); break;
-                case QUADRIC:  filter.calculate(agg::image_filter_quadric(), norm); break;
-                case CATROM: filter.calculate(agg::image_filter_catrom(), norm); break;
-                case GAUSSIAN: filter.calculate(agg::image_filter_gaussian(), norm); break;
-                case BESSEL: filter.calculate(agg::image_filter_bessel(), norm); break;
-                case MITCHELL: filter.calculate(agg::image_filter_mitchell(), norm); break;
-                case SINC: filter.calculate(agg::image_filter_sinc(radius), norm); break;
-                case LANCZOS: filter.calculate(agg::image_filter_lanczos(radius), norm); break;
-                case BLACKMAN: filter.calculate(agg::image_filter_blackman(radius), norm); break;
-                }
-
-	typedef agg::span_image_filter_rgba<agg::rgba8, agg::order_rgba,
-	  interpolator_type> span_gen_type;
-	typedef agg::renderer_scanline_aa<renderer_base, span_gen_type> renderer_type;
-	span_gen_type sg(sa, rbufPad, background, interpolator, filter);
-	renderer_type ri(rb, sg);
-	agg::render_scanlines(ras, sl, ri);
 
+    case HANNING:
+    case HAMMING:
+    case HERMITE:
+      {
+        agg::image_filter_lut filter;
+        switch (interpolation) {
+          case HANNING:  filter.calculate(agg::image_filter_hanning(), norm); break;
+          case HAMMING:  filter.calculate(agg::image_filter_hamming(), norm); break;
+          case HERMITE:  filter.calculate(agg::image_filter_hermite(), norm); break;
+        }
+	if (resample)
+	  {
+	    typedef agg::span_image_resample_rgba_affine<img_accessor_type> span_gen_type;
+	    typedef agg::renderer_scanline_aa<renderer_base, span_alloc_type, span_gen_type> renderer_type;
+	    span_gen_type sg(ia, interpolator, filter);
+	    renderer_type ri(rb, sa, sg);
+	    agg::render_scanlines(ras, sl, ri);
+	  }
+	else
+	  {
+	    typedef agg::span_image_filter_rgba_2x2<img_accessor_type, interpolator_type> span_gen_type;
+	    typedef agg::renderer_scanline_aa<renderer_base, span_alloc_type, span_gen_type> renderer_type;
+	    span_gen_type sg(ia, interpolator, filter);
+	    renderer_type ri(rb, sa, sg);
+	    agg::render_scanlines(ras, sl, ri);
+	  }
+      }
+      break;
+    case BILINEAR:
+    case BICUBIC:
+    case SPLINE16:
+    case SPLINE36:
+    case KAISER:
+    case QUADRIC:
+    case CATROM:
+    case GAUSSIAN:
+    case BESSEL:
+    case MITCHELL:
+    case SINC:
+    case LANCZOS:
+    case BLACKMAN:
+      {
+        agg::image_filter_lut filter;
+        switch(interpolation)
+          {
+          case BILINEAR:  filter.calculate(agg::image_filter_bilinear(), norm); break;
+          case BICUBIC:  filter.calculate(agg::image_filter_bicubic(), norm); break;
+          case SPLINE16:  filter.calculate(agg::image_filter_spline16(), norm); break;
+          case SPLINE36:  filter.calculate(agg::image_filter_spline36(), norm); break;
+          case KAISER:  filter.calculate(agg::image_filter_kaiser(), norm); break;
+          case QUADRIC:  filter.calculate(agg::image_filter_quadric(), norm); break;
+          case CATROM: filter.calculate(agg::image_filter_catrom(), norm); break;
+          case GAUSSIAN: filter.calculate(agg::image_filter_gaussian(), norm); break;
+          case BESSEL: filter.calculate(agg::image_filter_bessel(), norm); break;
+          case MITCHELL: filter.calculate(agg::image_filter_mitchell(), norm); break;
+          case SINC: filter.calculate(agg::image_filter_sinc(radius), norm); break;
+          case LANCZOS: filter.calculate(agg::image_filter_lanczos(radius), norm); break;
+          case BLACKMAN: filter.calculate(agg::image_filter_blackman(radius), norm); break;
+          }
+	if (resample)
+	  {
+	    typedef agg::span_image_resample_rgba_affine<img_accessor_type> span_gen_type;
+	    typedef agg::renderer_scanline_aa<renderer_base, span_alloc_type, span_gen_type> renderer_type;
+	    span_gen_type sg(ia, interpolator, filter);
+	    renderer_type ri(rb, sa, sg);
+	    agg::render_scanlines(ras, sl, ri);
+	  }
+	else
+	  {
+	    typedef agg::span_image_filter_rgba<img_accessor_type, interpolator_type> span_gen_type;
+	    typedef agg::renderer_scanline_aa<renderer_base, span_alloc_type, span_gen_type> renderer_type;
+	    span_gen_type sg(ia, interpolator, filter);
+	    renderer_type ri(rb, sa, sg);
+	    agg::render_scanlines(ras, sl, ri);
+	  }
       }
       break;
 
@@ -537,6 +550,20 @@ Image::get_size(const Py::Tuple& args) {
 
 }
 
+char Image::get_resample__doc__[] =
+"get_resample()\n"
+"\n"
+"Get the resample flag."
+;
+
+Py::Object
+Image::get_resample(const Py::Tuple& args) {
+  _VERBOSE("Image::get_resample");
+
+  args.verify_length(0);
+  return Py::Int((int)resample);
+}
+
 char Image::get_size_out__doc__[] =
 "numrows, numcols = get_size()\n"
 "\n"
@@ -545,7 +572,7 @@ char Image::get_size_out__doc__[] =
 
 Py::Object
 Image::get_size_out(const Py::Tuple& args) {
-  _VERBOSE("Image::get_size");
+  _VERBOSE("Image::get_size_out");
 
   args.verify_length(0);
 
@@ -600,95 +627,22 @@ Image::set_interpolation(const Py::Tuple& args) {
 
 }
 
-
-
-// this code is heavily adapted from the paint license, which is in
-// the file paint.license (BSD compatible) included in this
-// distribution.  TODO, add license file to MANIFEST.in and CVS
-char Image::write_png__doc__[] =
-"write_png(fname)\n"
+char Image::set_resample__doc__[] =
+"set_resample(boolean)\n"
 "\n"
-"Write the image to filename fname as png\n"
+"Set the resample flag."
 ;
+
 Py::Object
-Image::write_png(const Py::Tuple& args)
-{
-  //small memory leak in this function - JDH 2004-06-08
-  _VERBOSE("Image::write_png");
-
+Image::set_resample(const Py::Tuple& args) {
+  _VERBOSE("Image::set_resample");
   args.verify_length(1);
-
-  std::pair<agg::int8u*,bool> bufpair = _get_output_buffer();
-
-  std::string fileName = Py::String(args[0]);
-  const char *file_name = fileName.c_str();
-  FILE *fp;
-  png_structp png_ptr;
-  png_infop info_ptr;
-  struct        png_color_8_struct sig_bit;
-  png_uint_32 row=0;
-
-  //todo: allocate on heap
-  png_bytep row_pointers[rowsOut];
-  for (row = 0; row < rowsOut; ++row)
-    row_pointers[row] = bufpair.first + row * colsOut * 4;
-
-  fp = fopen(file_name, "wb");
-  if (fp == NULL) {
-    if (bufpair.second) delete [] bufpair.first;
-    throw Py::RuntimeError(Printf("Could not open file %s", file_name).str());
-  }
-
-
-  png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
-  if (png_ptr == NULL) {
-    if (bufpair.second) delete [] bufpair.first;
-    fclose(fp);
-    throw Py::RuntimeError("Could not create write struct");
-  }
-
-  info_ptr = png_create_info_struct(png_ptr);
-  if (info_ptr == NULL) {
-    if (bufpair.second) delete [] bufpair.first;
-    fclose(fp);
-    png_destroy_write_struct(&png_ptr, &info_ptr);
-    throw Py::RuntimeError("Could not create info struct");
-  }
-
-  if (setjmp(png_ptr->jmpbuf)) {
-    if (bufpair.second) delete [] bufpair.first;
-    fclose(fp);
-    png_destroy_write_struct(&png_ptr, &info_ptr);
-    throw Py::RuntimeError("Error building image");
-  }
-
-  png_init_io(png_ptr, fp);
-  png_set_IHDR(png_ptr, info_ptr,
-	       colsOut, rowsOut, 8,
-	       PNG_COLOR_TYPE_RGB_ALPHA, PNG_INTERLACE_NONE,
-	       PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);
-
-  // this a a color image!
-  sig_bit.gray = 0;
-  sig_bit.red = 8;
-  sig_bit.green = 8;
-  sig_bit.blue = 8;
-  /* if the image has an alpha channel then */
-  sig_bit.alpha = 8;
-  png_set_sBIT(png_ptr, info_ptr, &sig_bit);
-
-  png_write_info(png_ptr, info_ptr);
-  png_write_image(png_ptr, row_pointers);
-  png_write_end(png_ptr, info_ptr);
-  png_destroy_write_struct(&png_ptr, &info_ptr);
-  fclose(fp);
-
-  if (bufpair.second) delete [] bufpair.first;
+  int flag = Py::Int(args[0]);
+  resample = (bool)flag;
   return Py::Object();
 }
 
 
-
 char Image::set_aspect__doc__[] =
 "set_aspect(scheme)\n"
 "\n"
@@ -719,17 +673,19 @@ Image::init_type() {
   add_varargs_method( "apply_scaling",	&Image::apply_scaling, Image::apply_scaling__doc__);
   add_varargs_method( "apply_translation", &Image::apply_translation, Image::apply_translation__doc__);
   add_keyword_method( "as_rgba_str", &Image::as_rgba_str, Image::as_rgba_str__doc__);
-  add_varargs_method( "buffer_argb32", &Image::buffer_argb32, Image::buffer_argb32__doc__);
+  add_varargs_method( "color_conv", &Image::color_conv, Image::color_conv__doc__);
   add_varargs_method( "buffer_rgba", &Image::buffer_rgba, Image::buffer_rgba__doc__);
   add_varargs_method( "get_aspect", &Image::get_aspect, Image::get_aspect__doc__);
   add_varargs_method( "get_interpolation", &Image::get_interpolation, Image::get_interpolation__doc__);
+  add_varargs_method( "get_resample", &Image::get_resample, Image::get_resample__doc__);
   add_varargs_method( "get_size", &Image::get_size, Image::get_size__doc__);
   add_varargs_method( "get_size_out", &Image::get_size_out, Image::get_size_out__doc__);
   add_varargs_method( "reset_matrix", &Image::reset_matrix, Image::reset_matrix__doc__);
+  add_varargs_method( "get_matrix", &Image::get_matrix, Image::get_matrix__doc__);
   add_keyword_method( "resize", &Image::resize, Image::resize__doc__);
   add_varargs_method( "set_interpolation", &Image::set_interpolation, Image::set_interpolation__doc__);
+  add_varargs_method( "set_resample", &Image::set_resample, Image::set_resample__doc__);
   add_varargs_method( "set_aspect", &Image::set_aspect, Image::set_aspect__doc__);
-  add_varargs_method( "write_png", &Image::write_png, Image::write_png__doc__);
   add_varargs_method( "set_bg", &Image::set_bg, Image::set_bg__doc__);
   add_varargs_method( "flipud_out", &Image::flipud_out, Image::flipud_out__doc__);
   add_varargs_method( "flipud_in", &Image::flipud_in, Image::flipud_in__doc__);
@@ -783,11 +739,14 @@ _image_module::from_images(const Py::Tuple& args) {
   renderer_base rb(pixf);
 
 
+  //clear the background of the rendering buffer with alpha 1 and the
+  //gtkagg screen noise problem in figimage_demo.py goes away -- see
+  //comment backend_gtkagg.py _render_figure method JDH
+  //rb.clear(agg::rgba(1, 1, 1, 1));
+
   for (size_t imnum=0; imnum< N; imnum++) {
     tup = Py::Tuple(tups[imnum]);
     Image* thisim = static_cast<Image*>(tup[0].ptr());
-    if (imnum==0)
-      rb.clear(thisim->bg);
     ox = Py::Int(tup[1]);
     oy = Py::Int(tup[2]);
 
@@ -796,7 +755,7 @@ _image_module::from_images(const Py::Tuple& args) {
       for (size_t i=0; i<thisim->colsOut; i++) {
 	thisx = i+ox;
 	thisy = j+oy;
-	if (thisx<0 || thisx>=numcols || thisy<0 || thisy>=numrows) {
+	if (thisx>=numcols || thisy>=numrows) {
 	  ind +=4;
 	  continue;
 	}
@@ -818,119 +777,10 @@ _image_module::from_images(const Py::Tuple& args) {
 }
 
 
-
-char _image_module_readpng__doc__[] =
-"readpng(fname)\n"
-"\n"
-"Load an image from png file into a numerix array of MxNx4 uint8";
-Py::Object
-_image_module::readpng(const Py::Tuple& args) {
-
-  args.verify_length(1);
-  std::string fname = Py::String(args[0]);
-
-  png_byte header[8];	// 8 is the maximum size that can be checked
-
-  FILE *fp = fopen(fname.c_str(), "rb");
-  if (!fp)
-    throw Py::RuntimeError(Printf("_image_module::readpng could not open PNG file %s for reading", fname.c_str()).str());
-
-  fread(header, 1, 8, fp);
-  if (png_sig_cmp(header, 0, 8))
-    throw Py::RuntimeError("_image_module::readpng: file not recognized as a PNG file");
-
-
-  /* initialize stuff */
-  png_structp png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
-
-  if (!png_ptr)
-    throw Py::RuntimeError("_image_module::readpng:  png_create_read_struct failed");
-
-  png_infop info_ptr = png_create_info_struct(png_ptr);
-  if (!info_ptr)
-    throw Py::RuntimeError("_image_module::readpng:  png_create_info_struct failed");
-
-  if (setjmp(png_jmpbuf(png_ptr)))
-    throw Py::RuntimeError("_image_module::readpng:  error during init_io");
-
-  png_init_io(png_ptr, fp);
-  png_set_sig_bytes(png_ptr, 8);
-
-  png_read_info(png_ptr, info_ptr);
-
-  png_uint_32 width = info_ptr->width;
-  png_uint_32 height = info_ptr->height;
-
-  // convert misc color types to rgb for simplicity
-  if (info_ptr->color_type == PNG_COLOR_TYPE_GRAY ||
-      info_ptr->color_type == PNG_COLOR_TYPE_GRAY_ALPHA)
-    png_set_gray_to_rgb(png_ptr);
-  else if (info_ptr->color_type == PNG_COLOR_TYPE_PALETTE)
-    png_set_palette_to_rgb(png_ptr);
-
-
-  int bit_depth = info_ptr->bit_depth;
-  if (bit_depth == 16)  png_set_strip_16(png_ptr);
-
-
-  png_set_interlace_handling(png_ptr);
-  png_read_update_info(png_ptr, info_ptr);
-
-  bool rgba = info_ptr->color_type == PNG_COLOR_TYPE_RGBA;
-  if ( (info_ptr->color_type != PNG_COLOR_TYPE_RGB) && !rgba) {
-    std::cerr << "Found color type " << (int)info_ptr->color_type  << std::endl;
-    throw Py::RuntimeError("_image_module::readpng: cannot handle color_type");
-  }
-
-  /* read file */
-  if (setjmp(png_jmpbuf(png_ptr)))
-    throw Py::RuntimeError("_image_module::readpng: error during read_image");
-
-  png_bytep row_pointers[height];
-
-  for (png_uint_32 row = 0; row < height; row++)
-    row_pointers[row] = new png_byte[png_get_rowbytes(png_ptr,info_ptr)];
-
-  png_read_image(png_ptr, row_pointers);
-
-
-
-  int dimensions[3];
-  dimensions[0] = height;  //numrows
-  dimensions[1] = width;   //numcols
-  dimensions[2] = 4;
-
-  PyArrayObject *A = (PyArrayObject *) PyArray_FromDims(3, dimensions, PyArray_FLOAT);
-
-
-  for (png_uint_32 y = 0; y < height; y++) {
-    png_byte* row = row_pointers[y];
-    for (png_uint_32 x = 0; x < width; x++) {
-
-      png_byte* ptr = (rgba) ? &(row[x*4]) : &(row[x*3]);
-      size_t offset = y*A->strides[0] + x*A->strides[1];
-      //if ((y<10)&&(x==10)) std::cout << "r = " << ptr[0] << " " << ptr[0]/255.0 << std::endl;
-      *(float*)(A->data + offset + 0*A->strides[2]) = ptr[0]/255.0;
-      *(float*)(A->data + offset + 1*A->strides[2]) = ptr[1]/255.0;
-      *(float*)(A->data + offset + 2*A->strides[2]) = ptr[2]/255.0;
-      *(float*)(A->data + offset + 3*A->strides[2]) = rgba ? ptr[3]/255.0 : 1.0;
-    }
-  }
-
-  //free the png memory
-  png_read_end(png_ptr, info_ptr);
-  png_destroy_read_struct(&png_ptr, &info_ptr, png_infopp_NULL);
-  fclose(fp);
-  for (png_uint_32 row = 0; row < height; row++)
-    delete [] row_pointers[row];
-  return Py::asObject((PyObject*)A);
-}
-
-
 char _image_module_fromarray__doc__[] =
 "fromarray(A, isoutput)\n"
 "\n"
-"Load the image from a Numeric or numarray array\n"
+"Load the image from a numpy array\n"
 "By default this function fills the input buffer, which can subsequently\n"
 "be resampled using resize.  If isoutput=1, fill the output buffer.\n"
 "This is used to support raw pixel images w/o resampling"
@@ -1039,7 +889,7 @@ _image_module::fromarray(const Py::Tuple& args) {
 char _image_module_fromarray2__doc__[] =
 "fromarray2(A, isoutput)\n"
 "\n"
-"Load the image from a Numeric or numarray array\n"
+"Load the image from a numpy array\n"
 "By default this function fills the input buffer, which can subsequently\n"
 "be resampled using resize.  If isoutput=1, fill the output buffer.\n"
 "This is used to support raw pixel images w/o resampling"
@@ -1161,7 +1011,8 @@ _image_module::frombyte(const Py::Tuple& args) {
   int isoutput = Py::Int(args[1]);
 
   PyArrayObject *A = (PyArrayObject *) PyArray_ContiguousFromObject(x.ptr(), PyArray_UBYTE, 3, 3);
-
+  if (A == NULL)
+      throw Py::ValueError("Array must have 3 dimensions");
   if (A->dimensions[2]<3 || A->dimensions[2]>4)
       throw Py::ValueError("Array dimension 3 must have size 3 or 4");
 
@@ -1184,10 +1035,10 @@ _image_module::frombyte(const Py::Tuple& args) {
   const size_t N = imo->rowsIn * imo->colsIn * imo->BPP;
   size_t i = 0;
   if (A->dimensions[2] == 4) {
-      std::memmove(buffer, arrbuf, N);
+      memmove(buffer, arrbuf, N);
   } else {
       while (i < N) {
-          std::memmove(buffer, arrbuf, 3);
+          memmove(buffer, arrbuf, 3);
           buffer += 3;
           arrbuf += 3;
           *buffer++ = 255;
@@ -1244,22 +1095,22 @@ _image_module::frombuffer(const Py::Tuple& args) {
 
   imo->rowsIn = y;
   imo->colsIn = x;
-  size_t NUMBYTES(imo->colsIn * imo->rowsIn * imo->BPP);
+  Py_ssize_t NUMBYTES(imo->colsIn * imo->rowsIn * imo->BPP);
 
-  int buflen;
+  Py_ssize_t buflen;
   const agg::int8u *rawbuf;
   if (PyObject_AsReadBuffer(bufin, reinterpret_cast<const void**>(&rawbuf), &buflen) != 0)
     throw Py::ValueError("Cannot get buffer from object.");
 
   // Check buffer is required size.
-  if ((size_t)buflen != NUMBYTES)
+  if (buflen != NUMBYTES)
     throw Py::ValueError("Buffer length must be width * height * 4.");
 
   // Copy from input buffer to new buffer for agg.
   agg::int8u* buffer = new agg::int8u[NUMBYTES];
   if (buffer==NULL) //todo: also handle allocation throw
     throw Py::MemoryError("_image_module::frombuffer could not allocate memory");
-  std::memmove(buffer, rawbuf, NUMBYTES);
+  memmove(buffer, rawbuf, NUMBYTES);
 
   if (isoutput) {
     // make the output buffer point to the input buffer
@@ -1278,7 +1129,7 @@ _image_module::frombuffer(const Py::Tuple& args) {
     imo->rbufIn->attach(buffer, imo->colsIn, imo->rowsIn, imo->colsIn*imo->BPP);
   }
 
-  return Py::Object();
+  return Py::asObject(imo);
 }
 
 
@@ -1319,7 +1170,7 @@ _image_module::pcolor(const Py::Tuple& args) {
   if (rows == 0 || cols ==0)
       throw Py::ValueError("Cannot scale to zero size");
 
-  // Get numeric arrays
+  // Get numpy arrays
   PyArrayObject *x = (PyArrayObject *) PyArray_ContiguousFromObject(xp.ptr(), PyArray_FLOAT, 1, 1);
   if (x == NULL)
       throw Py::ValueError("x is of incorrect type (wanted 1D float)");
@@ -1371,8 +1222,8 @@ _image_module::pcolor(const Py::Tuple& args) {
   // Create output
   Image* imo = new Image;
   imo->rowsIn = rows;
-  imo->rowsOut = rows;
   imo->colsIn = cols;
+  imo->rowsOut = rows;
   imo->colsOut = cols;
   size_t NUMBYTES(rows * cols * 4);
   agg::int8u *buffer = new agg::int8u[NUMBYTES];
@@ -1406,7 +1257,7 @@ _image_module::pcolor(const Py::Tuple& args) {
       while(xs2 != xl && xo > xm) {
           xs1 = xs2;
           xs2 = xs1+1;
-          xm = 0.5*(*xs1 + *xs2);
+          xm = 0.5f*(*xs1 + *xs2);
           j++;
       }
       *colstart = j - j_last;
@@ -1419,7 +1270,7 @@ _image_module::pcolor(const Py::Tuple& args) {
       while(ys2 != yl && yo > ym) {
           ys1 = ys2;
           ys2 = ys1+1;
-          ym = 0.5*(*ys1 + *ys2);
+          ym = 0.5f*(*ys1 + *ys2);
           j++;
       }
       *rowstart = j - j_last;
@@ -1434,7 +1285,7 @@ _image_module::pcolor(const Py::Tuple& args) {
   size_t rowsize(cols*4);
   rowstart = rowstarts;
   agg::int8u * position = buffer;
-  agg::int8u * oldposition;
+  agg::int8u * oldposition = NULL;
   start = reinterpret_cast<unsigned char*>(d->data);
   for(i=0;i<rows;i++,rowstart++)
   {
@@ -1467,6 +1318,227 @@ _image_module::pcolor(const Py::Tuple& args) {
   return Py::asObject(imo);
 }
 
+void _bin_indices(int *irows, int nrows, double *y, int ny,
+                                            double sc, double offs)
+{
+    int i;
+    if (sc*(y[ny-1] - y[0]) > 0)
+    {
+        int ii = 0;
+        int iilast = ny-1;
+        int iy0 = (int)floor(sc * (y[ii]  - offs));
+        int iy1 = (int)floor(sc * (y[ii+1]  - offs));
+        for (i=0; i<nrows && i<iy0; i++) {
+            irows[i] = -1;
+        }
+        for (; i<nrows; i++) {
+            while (i > iy1 && ii < iilast) {
+                ii++;
+                iy0 = iy1;
+                iy1 = (int)floor(sc * (y[ii+1] - offs));
+            }
+            if (i >= iy0 && i <= iy1) irows[i] = ii;
+            else break;
+        }
+        for (; i<nrows; i++) {
+            irows[i] = -1;
+        }
+    }
+    else
+    {
+        int iilast = ny-1;
+        int ii = iilast;
+        int iy0 = (int)floor(sc * (y[ii]  - offs));
+        int iy1 = (int)floor(sc * (y[ii-1]  - offs));
+        for (i=0; i<nrows && i<iy0; i++) {
+            irows[i] = -1;
+        }
+        for (; i<nrows; i++) {
+            while (i > iy1 && ii > 1) {
+                ii--;
+                iy0 = iy1;
+                iy1 = (int)floor(sc * (y[ii-1] - offs));
+            }
+            if (i >= iy0 && i <= iy1) irows[i] = ii-1;
+            else break;
+        }
+        for (; i<nrows; i++) {
+            irows[i] = -1;
+        }
+    }
+}
+
+char __image_module_pcolor2__doc__[] =
+"pcolor2(x, y, data, rows, cols, bounds, bg)\n"
+"\n"
+"Generate a pseudo-color image from data on a non-uniform grid\n"
+"specified by its cell boundaries.\n"
+"bounds = (x_left, x_right, y_bot, y_top)\n"
+"bg = ndarray of 4 uint8 representing background rgba\n"
+;
+Py::Object
+_image_module::pcolor2(const Py::Tuple& args) {
+    _VERBOSE("_image_module::pcolor2");
+
+    if (args.length() != 7)
+        throw Py::TypeError("Incorrect number of arguments (6 expected)");
+
+    Py::Object xp = args[0];
+    Py::Object yp = args[1];
+    Py::Object dp = args[2];
+    int rows = Py::Int(args[3]);
+    int cols = Py::Int(args[4]);
+    Py::Tuple bounds = args[5];
+    Py::Object bgp = args[6];
+
+    if (bounds.length() !=4)
+        throw Py::TypeError("Incorrect number of bounds (4 expected)");
+    double x_left = Py::Float(bounds[0]);
+    double x_right = Py::Float(bounds[1]);
+    double y_bot = Py::Float(bounds[2]);
+    double y_top = Py::Float(bounds[3]);
+
+    // Check we have something to output to
+    if (rows == 0 || cols ==0)
+        throw Py::ValueError("rows or cols is zero; there are no pixels");
+
+    // Get numpy arrays
+    PyArrayObject *x = (PyArrayObject *) PyArray_ContiguousFromObject(xp.ptr(),
+                                                          PyArray_DOUBLE, 1, 1);
+    if (x == NULL)
+        throw Py::ValueError("x is of incorrect type (wanted 1D double)");
+    PyArrayObject *y = (PyArrayObject *) PyArray_ContiguousFromObject(yp.ptr(),
+                                                          PyArray_DOUBLE, 1, 1);
+    if (y == NULL) {
+        Py_XDECREF(x);
+        throw Py::ValueError("y is of incorrect type (wanted 1D double)");
+    }
+    PyArrayObject *d = (PyArrayObject *) PyArray_ContiguousFromObject(dp.ptr(),
+                                                          PyArray_UBYTE, 3, 3);
+    if (d == NULL) {
+        Py_XDECREF(x);
+        Py_XDECREF(y);
+        throw Py::ValueError("data is of incorrect type (wanted 3D uint8)");
+    }
+    if (d->dimensions[2] != 4) {
+        Py_XDECREF(x);
+        Py_XDECREF(y);
+        Py_XDECREF(d);
+        throw Py::ValueError("data must be in RGBA format");
+    }
+
+    // Check dimensions match
+    int nx = x->dimensions[0];
+    int ny = y->dimensions[0];
+    if (nx != d->dimensions[1]+1 || ny != d->dimensions[0]+1) {
+        Py_XDECREF(x);
+        Py_XDECREF(y);
+        Py_XDECREF(d);
+        throw Py::ValueError("data and axis bin boundary dimensions are incompatible");
+    }
+
+    PyArrayObject *bg = (PyArrayObject *) PyArray_ContiguousFromObject(bgp.ptr(),
+                                                          PyArray_UBYTE, 1, 1);
+    if (bg == NULL) {
+        Py_XDECREF(x);
+        Py_XDECREF(y);
+        Py_XDECREF(d);
+        throw Py::ValueError("bg is of incorrect type (wanted 1D uint8)");
+    }
+    if (bg->dimensions[0] != 4) {
+        Py_XDECREF(x);
+        Py_XDECREF(y);
+        Py_XDECREF(d);
+        Py_XDECREF(bg);
+        throw Py::ValueError("bg must be in RGBA format");
+    }
+
+
+    // Allocate memory for pointer arrays
+    int * irows = reinterpret_cast<int*>(PyMem_Malloc(sizeof(int)*rows));
+    if (irows == NULL) {
+        Py_XDECREF(x);
+        Py_XDECREF(y);
+        Py_XDECREF(d);
+        Py_XDECREF(bg);
+        throw Py::MemoryError("Cannot allocate memory for lookup table");
+    }
+    int * jcols = reinterpret_cast<int*>(PyMem_Malloc(sizeof(int*)*cols));
+    if (jcols == NULL) {
+        Py_XDECREF(x);
+        Py_XDECREF(y);
+        Py_XDECREF(d);
+        Py_XDECREF(bg);
+        PyMem_Free(irows);
+        throw Py::MemoryError("Cannot allocate memory for lookup table");
+    }
+
+    // Create output
+    Image* imo = new Image;
+    imo->rowsIn = rows;
+    imo->rowsOut = rows;
+    imo->colsIn = cols;
+    imo->colsOut = cols;
+    size_t NUMBYTES(rows * cols * 4);
+    agg::int8u *buffer = new agg::int8u[NUMBYTES];
+    if (buffer == NULL) {
+        Py_XDECREF(x);
+        Py_XDECREF(y);
+        Py_XDECREF(d);
+        Py_XDECREF(bg);
+        PyMem_Free(irows);
+        PyMem_Free(jcols);
+        throw Py::MemoryError("Could not allocate memory for image");
+    }
+
+    // Calculate the pointer arrays to map input x to output x
+    int i, j;
+    double *x0 = reinterpret_cast<double*>(x->data);
+    double *y0 = reinterpret_cast<double*>(y->data);
+    double sx = cols/(x_right - x_left);
+    double sy = rows/(y_top - y_bot);
+    _bin_indices(jcols, cols, x0, nx, sx, x_left);
+    _bin_indices(irows, rows, y0, ny, sy, y_bot);
+
+    // Copy data to output buffer
+    agg::int8u * position = buffer;
+    unsigned char *start = reinterpret_cast<unsigned char*>(d->data);
+    unsigned char *bgptr = reinterpret_cast<unsigned char*>(bg->data);
+    int s0 = d->strides[0];
+    int s1 = d->strides[1];
+
+    for (i=0; i<rows; i++)
+    {
+        for (j=0; j<cols; j++)
+        {
+            if (irows[i] == -1 || jcols[j] == -1) {
+                memcpy(position, bgptr, 4*sizeof(agg::int8u));
+            }
+            else {
+                memcpy(position, (start + s0*irows[i] + s1*jcols[j]),
+                                                  4*sizeof(agg::int8u));
+            }
+            position += 4;
+        }
+    }
+
+    // Attach output buffer to output buffer
+    imo->rbufOut = new agg::rendering_buffer;
+    imo->bufferOut = buffer;
+    imo->rbufOut->attach(imo->bufferOut, imo->colsOut, imo->rowsOut, imo->colsOut * imo->BPP);
+
+    Py_XDECREF(x);
+    Py_XDECREF(y);
+    Py_XDECREF(d);
+    Py_XDECREF(bg);
+    PyMem_Free(irows);
+    PyMem_Free(jcols);
+
+    return Py::asObject(imo);
+}
+
+
+
 #if defined(_MSC_VER)
 DL_EXPORT(void)
 #elif defined(__cplusplus)
@@ -1475,47 +1547,37 @@ DL_EXPORT(void)
 void
 #endif
 
-#ifdef NUMARRAY
-init_na_image(void) {
-  _VERBOSE("init_na_image");
-#else
-#   ifdef NUMERIC
-  init_nc_image(void) {
-    _VERBOSE("init_nc_image");
-#   else
-  init_ns_image(void) {
-    _VERBOSE("init_ns_image");
-#   endif
-#endif
+init_image(void) {
+  _VERBOSE("init_image");
 
-    static _image_module* _image = new _image_module;
+  static _image_module* _image = new _image_module;
 
-    import_array();
-    Py::Dict d = _image->moduleDictionary();
+  import_array();
+  Py::Dict d = _image->moduleDictionary();
 
-    d["NEAREST"] = Py::Int(Image::NEAREST);
-    d["BILINEAR"] = Py::Int(Image::BILINEAR);
-    d["BICUBIC"] = Py::Int(Image::BICUBIC);
-    d["SPLINE16"] = Py::Int(Image::SPLINE16);
-    d["SPLINE36"] = Py::Int(Image::SPLINE36);
-    d["HANNING"] = Py::Int(Image::HANNING);
-    d["HAMMING"] = Py::Int(Image::HAMMING);
-    d["HERMITE"] = Py::Int(Image::HERMITE);
-    d["KAISER"]   = Py::Int(Image::KAISER);
-    d["QUADRIC"]   = Py::Int(Image::QUADRIC);
-    d["CATROM"]  = Py::Int(Image::CATROM);
-    d["GAUSSIAN"]  = Py::Int(Image::GAUSSIAN);
-    d["BESSEL"]  = Py::Int(Image::BESSEL);
-    d["MITCHELL"]  = Py::Int(Image::MITCHELL);
-    d["SINC"]  = Py::Int(Image::SINC);
-    d["LANCZOS"]  = Py::Int(Image::LANCZOS);
-    d["BLACKMAN"] = Py::Int(Image::BLACKMAN);
+  d["NEAREST"] = Py::Int(Image::NEAREST);
+  d["BILINEAR"] = Py::Int(Image::BILINEAR);
+  d["BICUBIC"] = Py::Int(Image::BICUBIC);
+  d["SPLINE16"] = Py::Int(Image::SPLINE16);
+  d["SPLINE36"] = Py::Int(Image::SPLINE36);
+  d["HANNING"] = Py::Int(Image::HANNING);
+  d["HAMMING"] = Py::Int(Image::HAMMING);
+  d["HERMITE"] = Py::Int(Image::HERMITE);
+  d["KAISER"]   = Py::Int(Image::KAISER);
+  d["QUADRIC"]   = Py::Int(Image::QUADRIC);
+  d["CATROM"]  = Py::Int(Image::CATROM);
+  d["GAUSSIAN"]  = Py::Int(Image::GAUSSIAN);
+  d["BESSEL"]  = Py::Int(Image::BESSEL);
+  d["MITCHELL"]  = Py::Int(Image::MITCHELL);
+  d["SINC"]  = Py::Int(Image::SINC);
+  d["LANCZOS"]  = Py::Int(Image::LANCZOS);
+  d["BLACKMAN"] = Py::Int(Image::BLACKMAN);
 
-    d["ASPECT_FREE"] = Py::Int(Image::ASPECT_FREE);
-    d["ASPECT_PRESERVE"] = Py::Int(Image::ASPECT_PRESERVE);
+  d["ASPECT_FREE"] = Py::Int(Image::ASPECT_FREE);
+  d["ASPECT_PRESERVE"] = Py::Int(Image::ASPECT_PRESERVE);
 
 
-  }
+}
 
 
 
