@@ -1,6 +1,9 @@
-#include <sstream>
 #include "ft2font.h"
 #include "mplutils.h"
+#include <sstream>
+
+#define PY_ARRAY_TYPES_PREFIX NumPy
+#include "numpy/arrayobject.h"
 
 #define FIXED_MAJOR(val) (*((short *) &val+1))
 #define FIXED_MINOR(val) (*((short *) &val+0))
@@ -8,9 +11,9 @@
 /**
  To improve the hinting of the fonts, this code uses a hack
  presented here:
- 
+
  http://antigrain.com/research/font_rasterization/index.html
- 
+
  The idea is to limit the effect of hinting in the x-direction, while
  preserving hinting in the y-direction.  Since freetype does not
  support this directly, the dpi in the x-direction is set higher than
@@ -20,7 +23,7 @@
  hinting, whereas the global transform does not, this is documented
  behavior of freetype, and therefore hopefully unlikely to change.
  The freetype 2 tutorial says:
- 
+
       NOTE: The transformation is applied to every glyph that is
       loaded through FT_Load_Glyph and is completely independent of
       any hinting process. This means that you won't get the same
@@ -42,7 +45,7 @@
 
 FT_Library _ft2Library;
 
-// FT2Image::FT2Image() : 
+// FT2Image::FT2Image() :
 //   _isDirty(true),
 //   _buffer(NULL),
 //   _width(0), _height(0),
@@ -53,7 +56,7 @@ FT_Library _ft2Library;
 
 FT2Image::FT2Image(unsigned long width, unsigned long height) :
   _isDirty(true),
-  _buffer(NULL), 
+  _buffer(NULL),
   _width(0), _height(0),
   _rgbCopy(NULL),
   _rgbaCopy(NULL) {
@@ -61,26 +64,28 @@ FT2Image::FT2Image(unsigned long width, unsigned long height) :
   resize(width, height);
 }
 
-FT2Image::~FT2Image() { 
+FT2Image::~FT2Image() {
   _VERBOSE("FT2Image::~FT2Image");
-  delete [] _buffer; 
-  _buffer=NULL; 
+  delete [] _buffer;
+  _buffer=NULL;
   delete _rgbCopy;
   delete _rgbaCopy;
 }
 
-void FT2Image::resize(unsigned long width, unsigned long height) {
+void FT2Image::resize(long width, long height) {
+  if (width < 0) width = 1;
+  if (height < 0) height = 1;
   size_t numBytes = width*height;
 
-  if (width != _width || height != _height) {
+  if ((unsigned long)width != _width || (unsigned long)height != _height) {
     if (numBytes > _width*_height) {
       delete [] _buffer;
       _buffer = NULL;
       _buffer = new unsigned char [numBytes];
     }
 
-    _width = width;
-    _height = height;
+    _width = (unsigned long)width;
+    _height = (unsigned long)height;
   }
 
   memset(_buffer, 0, numBytes);
@@ -151,10 +156,9 @@ FT2Image::py_write_bitmap(const Py::Tuple & args) {
 }
 
 void
-FT2Image::draw_rect(unsigned long x0, unsigned long y0, 
+FT2Image::draw_rect(unsigned long x0, unsigned long y0,
 		    unsigned long x1, unsigned long y1) {
-  if ( x0<0 || y0<0 || x1<0 || y1<0 ||
-       x0>_width || x1>_width ||
+  if ( x0>_width || x1>_width ||
        y0>_height || y1>_height )
     throw Py::ValueError("Rect coords outside image bounds");
 
@@ -195,12 +199,12 @@ FT2Image::py_draw_rect(const Py::Tuple & args) {
   return Py::Object();
 }
 
-void FT2Image::draw_rect_filled(unsigned long x0, unsigned long y0, 
+void FT2Image::draw_rect_filled(unsigned long x0, unsigned long y0,
 				unsigned long x1, unsigned long y1) {
-  x0 = CLAMP(x0, 0, _width);
-  y0 = CLAMP(y0, 0, _height);
-  x1 = CLAMP(x1, 0, _width);
-  y1 = CLAMP(y1, 0, _height);
+  x0 = std::min(x0, _width);
+  y0 = std::min(y0, _height);
+  x1 = std::min(x1, _width);
+  y1 = std::min(y1, _height);
 
   for (size_t j=y0; j<y1+1; j++) {
     for (size_t i=x0; i<x1+1; i++) {
@@ -209,7 +213,7 @@ void FT2Image::draw_rect_filled(unsigned long x0, unsigned long y0,
   }
 
   _isDirty = true;
-}  
+}
 
 char FT2Image::draw_rect_filled__doc__[] =
 "draw_rect_filled(x0, y0, x1, y1)\n"
@@ -245,11 +249,46 @@ FT2Image::py_as_str(const Py::Tuple & args) {
   args.verify_length(0);
 
   return Py::asObject
-    (PyString_FromStringAndSize((const char *)_buffer, 
+    (PyString_FromStringAndSize((const char *)_buffer,
 				_width*_height)
      );
 }
 
+char FT2Image::as_array__doc__[] =
+"x = image.as_array()\n"
+"\n"
+"Return the image buffer as a width x height numpy array of ubyte \n"
+"\n"
+;
+Py::Object
+FT2Image::py_as_array(const Py::Tuple & args) {
+  _VERBOSE("FT2Image::as_array");
+  args.verify_length(0);
+
+  npy_intp dimensions[2];
+  dimensions[0] = get_height();  //numrows
+  dimensions[1] = get_width();   //numcols
+
+
+  PyArrayObject *A = (PyArrayObject *) PyArray_SimpleNewFromData(2, dimensions, PyArray_UBYTE, _buffer);
+
+  /*
+
+  PyArrayObject *A = (PyArrayObject *) PyArray_FromDims(2, dimensions, PyArray_UBYTE);
+
+
+  unsigned char *src		= _buffer;
+  unsigned char *src_end	= src + (dimensions[0] * dimensions[1]);
+  unsigned char *dst		= (unsigned char *)A->data;
+
+  while (src != src_end) {
+    *dst++ = *src++;
+  }
+  */
+
+  return Py::asObject((PyObject*)A);
+}
+
 void FT2Image::makeRgbCopy() {
   if (!_isDirty)
     return;
@@ -284,7 +323,7 @@ FT2Image::py_as_rgb_str(const Py::Tuple & args) {
   args.verify_length(0);
 
   makeRgbCopy();
-  
+
   return _rgbCopy->py_as_str(args);
 }
 
@@ -321,7 +360,7 @@ FT2Image::py_as_rgba_str(const Py::Tuple & args) {
   args.verify_length(0);
 
   makeRgbaCopy();
-  
+
   return _rgbaCopy->py_as_str(args);
 }
 
@@ -671,7 +710,7 @@ FT2Font::FT2Font(std::string facefile) :
   }
 
   // set a default fontsize 12 pt at 72dpi
-#ifdef VERTICAL_HINTING  
+#ifdef VERTICAL_HINTING
   error = FT_Set_Char_Size( face, 12 * 64, 0, 72 * HORIZ_HINTING, 72 );
   static FT_Matrix transform = { 65536 / HORIZ_HINTING, 0, 0, 65536 };
   FT_Set_Transform( face, &transform, 0 );
@@ -749,10 +788,6 @@ FT2Font::~FT2Font()
   for (size_t i=0; i<glyphs.size(); i++) {
     FT_Done_Glyph( glyphs[i] );
   }
-
-  for (size_t i=0; i<gms.size(); i++) {
-    Py_DECREF(gms[i]);
-  }
 }
 
 int
@@ -792,12 +827,7 @@ FT2Font::clear(const Py::Tuple & args) {
     FT_Done_Glyph( glyphs[i] );
   }
 
-  for (size_t i=0; i<gms.size(); i++) {
-    Py_DECREF(gms[i]);
-  }
-
-  glyphs.resize(0);
-  gms.resize(0);
+  glyphs.clear();
 
   return Py::Object();
 }
@@ -829,7 +859,7 @@ FT2Font::set_size(const Py::Tuple & args) {
   int error = FT_Set_Char_Size( face, (long)(ptsize * 64), 0,
 				(unsigned int)dpi,
 				(unsigned int)dpi );
-#endif  
+#endif
   if (error)
     throw Py::RuntimeError("Could not set the fontsize");
   return Py::Object();
@@ -1020,26 +1050,6 @@ FT2Font::set_text(const Py::Tuple & args, const Py::Dict & kwargs) {
   return xys;
 }
 
-
-char FT2Font::get_glyph__doc__[] =
-"get_glyph(num)\n"
-"\n"
-"Return the glyph object with num num\n"
-;
-Py::Object
-FT2Font::get_glyph(const Py::Tuple & args){
-  _VERBOSE("FT2Font::get_glyph");
-
-  args.verify_length(1);
-  int num = Py::Int(args[0]);
-
-  if ( (size_t)num >= gms.size())
-    throw Py::ValueError("Glyph index out of range");
-
-  Py_INCREF(gms[num]);
-  return Py::asObject(gms[num]);
-}
-
 char FT2Font::get_num_glyphs__doc__[] =
 "get_num_glyphs()\n"
 "\n"
@@ -1078,7 +1088,7 @@ FT2Font::load_char(const Py::Tuple & args, const Py::Dict & kwargs) {
   long charcode = Py::Long(args[0]), flags = Py::Long(FT_LOAD_FORCE_AUTOHINT);
   if (kwargs.hasKey("flags"))
     flags = Py::Long(kwargs["flags"]);
-  
+
   int error = FT_Load_Char( face, (unsigned long)charcode, flags);
 
   if (error)
@@ -1093,9 +1103,7 @@ FT2Font::load_char(const Py::Tuple & args, const Py::Dict & kwargs) {
   size_t num = glyphs.size();  //the index into the glyphs list
   glyphs.push_back(thisGlyph);
   Glyph* gm = new Glyph(face, thisGlyph, num);
-  gms.push_back(gm);
-  Py_INCREF(gm); //todo: refcount correct?
-  return Py::asObject( gm);
+  return Py::asObject(gm);
 }
 
 char FT2Font::get_width_height__doc__[] =
@@ -1238,7 +1246,7 @@ char FT2Font::draw_glyph_to_bitmap__doc__[] =
 "draw_glyphs_to_bitmap.  This function is intended for people who\n"
 "want to render individual glyphs at precise locations, eg, a\n"
 "a glyph returned by load_char\n";
-;
+
 Py::Object
 FT2Font::draw_glyph_to_bitmap(const Py::Tuple & args) {
   _VERBOSE("FT2Font::draw_glyph_to_bitmap");
@@ -1659,7 +1667,7 @@ FT2Font::get_sfnt_table(const Py::Tuple & args) {
   }
 }
 
-char FT2Font::get_image__doc__ [] = 
+char FT2Font::get_image__doc__ [] =
   "get_image()\n"
   "\n"
   "Returns the underlying image buffer for this font object.\n";
@@ -1669,7 +1677,7 @@ FT2Font::get_image (const Py::Tuple &args) {
   if (image) {
     Py_XINCREF(image);
     return Py::asObject(image);
-  } 
+  }
   throw Py::RuntimeError("You must call .set_text() before .get_image()");
 }
 
@@ -1684,7 +1692,7 @@ FT2Font::attach_file (const Py::Tuple &args) {
   args.verify_length(1);
 
   std::string filename = Py::String(args[0]);
-  FT_Error error = 
+  FT_Error error =
     FT_Attach_File(face, filename.c_str());
 
   if (error) {
@@ -1727,6 +1735,8 @@ FT2Image::init_type() {
 		    FT2Image::draw_rect__doc__);
  add_varargs_method("draw_rect_filled", &FT2Image::py_draw_rect_filled,
 		    FT2Image::draw_rect_filled__doc__);
+ add_varargs_method("as_array", &FT2Image::py_as_array,
+		    FT2Image::as_array__doc__);
  add_varargs_method("as_str", &FT2Image::py_as_str,
 		    FT2Image::as_str__doc__);
  add_varargs_method("as_rgb_str", &FT2Image::py_as_rgb_str,
@@ -1763,8 +1773,6 @@ FT2Font::init_type() {
   add_varargs_method("get_xys", &FT2Font::get_xys,
 		     FT2Font::get_xys__doc__);
 
-  add_varargs_method("get_glyph", &FT2Font::get_glyph,
-		     FT2Font::get_glyph__doc__);
   add_varargs_method("get_num_glyphs", &FT2Font::get_num_glyphs,
 		     FT2Font::get_num_glyphs__doc__);
   add_keyword_method("load_char", &FT2Font::load_char,
@@ -1801,7 +1809,7 @@ FT2Font::init_type() {
 
   behaviors().supportGetattr();
   behaviors().supportSetattr();
-};
+}
 
 //todo add module docs strings
 
@@ -1866,6 +1874,7 @@ void
 initft2font(void)
 {
   static ft2font_module* ft2font = new ft2font_module;
+  import_array();
 
   Py::Dict d = ft2font->moduleDictionary();
   d["SCALABLE"] 	= Py::Int(FT_FACE_FLAG_SCALABLE);
@@ -1884,7 +1893,7 @@ initft2font(void)
   d["KERNING_DEFAULT"]  = Py::Int(FT_KERNING_DEFAULT);
   d["KERNING_UNFITTED"]  = Py::Int(FT_KERNING_UNFITTED);
   d["KERNING_UNSCALED"]  = Py::Int(FT_KERNING_UNSCALED);
-  
+
   d["LOAD_DEFAULT"]    	     = Py::Long(FT_LOAD_DEFAULT);
   d["LOAD_NO_SCALE"]   	     = Py::Long(FT_LOAD_NO_SCALE);
   d["LOAD_NO_HINTING"] 	     = Py::Long(FT_LOAD_NO_HINTING);
@@ -1894,7 +1903,7 @@ initft2font(void)
   d["LOAD_FORCE_AUTOHINT"]   = Py::Long(FT_LOAD_FORCE_AUTOHINT);
   d["LOAD_CROP_BITMAP"]      = Py::Long(FT_LOAD_CROP_BITMAP);
   d["LOAD_PEDANTIC"]         = Py::Long(FT_LOAD_PEDANTIC);
-  d["LOAD_IGNORE_GLOBAL_ADVANCE_WIDTH"] = 
+  d["LOAD_IGNORE_GLOBAL_ADVANCE_WIDTH"] =
     Py::Long(FT_LOAD_IGNORE_GLOBAL_ADVANCE_WIDTH);
   d["LOAD_NO_RECURSE"]       = Py::Long(FT_LOAD_NO_RECURSE);
   d["LOAD_IGNORE_TRANSFORM"] = Py::Long(FT_LOAD_IGNORE_TRANSFORM);
@@ -1917,5 +1926,6 @@ initft2font(void)
 }
 
 ft2font_module::~ft2font_module() {
+
   FT_Done_FreeType( _ft2Library );
 }
